'''
'''
#  Licensed to the Apache Software Foundation (ASF) under one
#  or more contributor license agreements.  See the NOTICE file
#  distributed with this work for additional information
#  regarding copyright ownership.  The ASF licenses this file
#  to you under the Apache License, Version 2.0 (the
#  "License"); you may not use this file except in compliance
#  with the License.  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

import os

# ----
# Setup Test
# ----
Test.Summary = '''
Test a case when the origin returns before the post is completed
'''
Test.ContinueOnFail = True

Test.GetTcpPort("upstream_port1")
Test.GetTcpPort("upstream_port2")
Test.GetTcpPort("upstream_port3")
Test.GetTcpPort("upstream_port4")
Test.GetTcpPort("upstream_port5")
Test.GetTcpPort("upstream_port6")

# ----
# Setup ATS
# ----
ts = Test.MakeATSProcess("ts", enable_tls=True, enable_cache=False)

# add ssl materials like key, certificates for the server
ts.addDefaultSSLFiles()

ts.Disk.remap_config.AddLines(
    [
        'map /one http://127.0.0.1:{0}'.format(Test.Variables.upstream_port1),
        'map /two http://127.0.0.1:{0}'.format(Test.Variables.upstream_port2),
        'map /three http://127.0.0.1:{0}'.format(Test.Variables.upstream_port3),
        'map /four http://127.0.0.1:{0}'.format(Test.Variables.upstream_port4),
        'map /five http://127.0.0.1:{0}'.format(Test.Variables.upstream_port5),
        'map /six http://127.0.0.1:{0}'.format(Test.Variables.upstream_port6),
    ])
ts.Disk.ssl_multicert_config.AddLine('dest_ip=* ssl_cert_name=server.pem ssl_key_name=server.key')
ts.Disk.records_config.update(
    {
        'proxy.config.ssl.server.cert.path': '{0}'.format(ts.Variables.SSLDir),
        'proxy.config.ssl.server.private_key.path': '{0}'.format(ts.Variables.SSLDir),
        'proxy.config.diags.debug.enabled': 0,
        # 'proxy.config.http2.initial_window_size_in': 2*16384, # Make a ludacrisly small window
        'proxy.config.diags.debug.tags': 'http',
    })

server1 = Test.Processes.Process(
    "server1", "bash -c '" + Test.TestDirectory + "/server1.sh {} outserver1'".format(Test.Variables.upstream_port1))
server2 = Test.Processes.Process(
    "server2", "bash -c '" + Test.TestDirectory + "/server1.sh {} outserver1'".format(Test.Variables.upstream_port2))
server3 = Test.Processes.Process(
    "server3", "bash -c '" + Test.TestDirectory + "/server1.sh {} outserver1'".format(Test.Variables.upstream_port3))
server4 = Test.Processes.Process(
    "server4", "bash -c '" + Test.TestDirectory + "/server1.sh {} outserver1'".format(Test.Variables.upstream_port4))
server5 = Test.Processes.Process(
    "server5", "bash -c '" + Test.TestDirectory + "/server1.sh {} outserver1'".format(Test.Variables.upstream_port5))
server6 = Test.Processes.Process(
    "server6", "bash -c '" + Test.TestDirectory + "/server1.sh {} outserver1'".format(Test.Variables.upstream_port6))

big_post_body = "0123456789" * 231070
big_post_body_file = open(os.path.join(Test.RunDirectory, "big_post_body"), "w")
big_post_body_file.write(big_post_body)
big_post_body_file.close()

# First two cases we are using curl with no means to delay the post body.  Even for the large body case, it looks like ATS processes the entire body before getting the response header
# The third case has an explicit multi-second sleep which ensures the early response path is exercised
test_run = Test.AddTestRun("http1.1 Post with small body early return")
test_run.Processes.Default.StartBefore(Test.Processes.ts)
test_run.Processes.Default.StartBefore(server1)
test_run.Processes.Default.Command = 'curl -v -o /dev/null --http1.1 -d "small body" -k https://127.0.0.1:{}/one'.format(
    ts.Variables.ssl_port)
test_run.Processes.Default.Streams.All = Testers.ContainsExpression("HTTP/1.1 420 Be Calm", "Receive the early response")
test_run.StillRunningAfter = ts
test_run.Processes.Default.ReturnCode = 0

test_run = Test.AddTestRun("http1.1 Post with large body early return")
test_run.Processes.Default.StartBefore(server2)
test_run.Processes.Default.Command = 'curl -H "Expect:" -v -o /dev/null --http1.1 -d @big_post_body -k https://127.0.0.1:{}/two'.format(
    ts.Variables.ssl_port)
test_run.Processes.Default.Streams.All = Testers.ContainsExpression("HTTP/1.1 420 Be Calm", "Receive the early response")
test_run.StillRunningAfter = ts
test_run.Processes.Default.ReturnCode = 0

test_run = Test.AddTestRun("http2 Post with large body, small window and early return")
test_run.Processes.Default.StartBefore(server3)
test_run.Processes.Default.Command = 'curl -v -o /dev/null --http2 -d @big_post_body -k https://127.0.0.1:{}/three'.format(
    ts.Variables.ssl_port)
test_run.Processes.Default.Streams.All = Testers.ContainsExpression("HTTP/2 420", "Receive the early response")
test_run.StillRunningAfter = ts
test_run.Processes.Default.ReturnCode = 0

client_out = Test.Disk.File("clientout")
client_out.Content = Testers.ExcludesExpression("0123456789", "The delayed body is not sent")
client_out.Content += Testers.ContainsExpression("HTTP/1.1 420 Be Calm", "Receive the early response")
client_out.Content += Testers.ContainsExpression("Connection: close", "ATS marks the client connection to close")

client_out2 = Test.Disk.File("clientout2")
client_out2.Content = Testers.ExcludesExpression("0123456789", "The delayed body is not sent")
client_out2.Content += Testers.ContainsExpression("HTTP/1.1 420 Be Calm", "Receive the early response")
client_out2.Content += Testers.ContainsExpression("Connection: close", "ATS marks the client connection to close")

client_out3 = Test.Disk.File("clientout3")
client_out3.Content = Testers.ExcludesExpression("0123456789", "The delayed body is not sent")
client_out3.Content += Testers.ContainsExpression("HTTP/1.1 420 Be Calm", "Receive the early response")
client_out3.Content += Testers.ContainsExpression("Connection: close", "ATS marks the client connection to close")

test_run = Test.AddTestRun("http1.1 Post with paused body")
test_run.Processes.Default.StartBefore(server4)
test_run.Setup.Copy("delay_client.sh")
test_run.Processes.Default.Command = "sh ./delay_client.sh {} clientout".format(ts.Variables.port)
test_run.StillRunningAfter = ts
test_run.Processes.Default.ReturnCode = 0

test_run = Test.AddTestRun("http1.1 Post with delayed and paused body")
test_run.Processes.Default.StartBefore(server5)
test_run.Setup.Copy("delay_client2.sh")
test_run.Processes.Default.Command = "sh ./delay_client2.sh {} clientout2".format(ts.Variables.port)
test_run.StillRunningAfter = ts
test_run.Processes.Default.ReturnCode = 0

test_run = Test.AddTestRun("http1.1 Post with paused body and no delay on server")
test_run.Processes.Default.StartBefore(server6)
test_run.Setup.Copy("delay_client3.sh")
test_run.Processes.Default.Command = "sh ./delay_client3.sh {} clientout3".format(ts.Variables.port)
test_run.StillRunningAfter = ts
test_run.Processes.Default.ReturnCode = 0



'''
Test per SNI server name selection of CA certs for validating cert sent by client.
'''
#  Licensed to the Apache Software Foundation (ASF) under one
#  or more contributor license agreements.  See the NOTICE file
#  distributed with this work for additional information
#  regarding copyright ownership.  The ASF licenses this file
#  to you under the Apache License, Version 2.0 (the
#  "License"); you may not use this file except in compliance
#  with the License.  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

Test.Summary = '''
Test per SNI server name selection of CA certs for validating cert sent by client.
'''

ts = Test.MakeATSProcess("ts", enable_tls=True)

server = Test.MakeOriginServer("server")

request_header = {"headers": "GET /xyz HTTP/1.1\r\nHost: example.com\r\n\r\n", "timestamp": "1469733493.993", "body": ""}
response_header = {"headers": "HTTP/1.1 200 OK\r\nConnection: close\r\n\r\n", "timestamp": "1469733493.993", "body": "yadayadayada"}
server.addResponse("sessionlog.json", request_header, response_header)

ts.addSSLfile("ssl/server.pem")
ts.addSSLfile("ssl/server.key")

ts.Setup.Copy("ssl/bbb-ca.pem", ts.Variables.CONFIGDIR)
ts.Setup.Copy("ssl/bbb-signed.key", ts.Variables.SSLDir)
ts.Setup.Copy("ssl/bbb-signed.pem", ts.Variables.SSLDir)
ts.Setup.Copy("ssl/aaa-ca.pem", ts.Variables.SSLDir)
ts.Setup.Copy("ssl/ccc-ca.pem", ts.Variables.SSLDir)

ts.Disk.records_config.update(
    {
        'proxy.config.diags.debug.enabled': 1,
        'proxy.config.diags.debug.tags': 'ssl',
        'proxy.config.ssl.server.cert.path': ts.Variables.SSLDir,
        'proxy.config.ssl.server.private_key.path': ts.Variables.SSLDir,
        'proxy.config.url_remap.pristine_host_hdr': 1,
        'proxy.config.ssl.client.certification_level': 2,
        'proxy.config.ssl.CA.cert.filename': f'{ts.Variables.SSLDir}/aaa-ca.pem',
        'proxy.config.ssl.TLSv1_3.enabled': 0
    })

ts.Disk.ssl_multicert_config.AddLine('ssl_cert_name=bbb-signed.pem ssl_key_name=bbb-signed.key')
ts.Disk.ssl_multicert_config.AddLine('dest_ip=* ssl_cert_name=server.pem ssl_key_name=server.key')

# Just map everything through to origin.  This test is concentrating on the user-agent side.
ts.Disk.remap_config.AddLine(f'map / http://127.0.0.1:{server.Variables.Port}/')

ts.Disk.sni_yaml.AddLines(
    [
        'sni:',
        '- fqdn: bbb.com',
        '  verify_client: STRICT',
        '  verify_client_ca_certs: bbb-ca.pem',
        '- fqdn: bbb-signed',
        '  verify_client: STRICT',
        '  verify_client_ca_certs: bbb-ca.pem',
        '- fqdn: ccc.com',
        '  verify_client: STRICT',
        '  verify_client_ca_certs:',
        f'    file: {ts.Variables.SSLDir}/ccc-ca.pem',
    ])

# Success test runs.

tr = Test.AddTestRun()
tr.Processes.Default.StartBefore(Test.Processes.ts)
tr.Processes.Default.StartBefore(server)
tr.StillRunningAfter = ts
tr.StillRunningAfter = server
tr.Processes.Default.Command = (
    "curl -v -k --tls-max 1.2  --cert {1}.pem --key {1}.key --resolve 'aaa.com:{0}:127.0.0.1'" + " https://aaa.com:{0}/xyz").format(
        ts.Variables.ssl_port, Test.TestDirectory + "/ssl/aaa-signed")
tr.Processes.Default.ReturnCode = 0
tr.Processes.Default.Streams.All = Testers.ExcludesExpression("error", "Check response")

tr = Test.AddTestRun()
tr.StillRunningAfter = ts
tr.StillRunningAfter = server
tr.Processes.Default.Command = (
    "curl -v -k --tls-max 1.2  --cert {1}.pem --key {1}.key --resolve 'bbb-signed:{0}:127.0.0.1'" +
    " https://bbb-signed:{0}/xyz").format(ts.Variables.ssl_port, Test.TestDirectory + "/ssl/bbb-signed")
tr.Processes.Default.ReturnCode = 0
tr.Processes.Default.Streams.All = Testers.ExcludesExpression("error", "Check response")

tr = Test.AddTestRun()
tr.StillRunningAfter = ts
tr.StillRunningAfter = server
tr.Processes.Default.Command = (
    "curl -v -k --tls-max 1.2  --cert {1}.pem --key {1}.key --resolve 'ccc.com:{0}:127.0.0.1'" + " https://ccc.com:{0}/xyz").format(
        ts.Variables.ssl_port, Test.TestDirectory + "/ssl/ccc-signed")
tr.Processes.Default.ReturnCode = 0
tr.Processes.Default.Streams.All = Testers.ExcludesExpression("error", "Check response")

# Failure test runs.

tr = Test.AddTestRun()
tr.StillRunningAfter = ts
tr.StillRunningAfter = server
tr.Processes.Default.Command = (
    "curl -v -k --tls-max 1.2  --cert {1}.pem --key {1}.key --resolve 'aaa.com:{0}:127.0.0.1'" + " https://aaa.com:{0}/xyz").format(
        ts.Variables.ssl_port, Test.TestDirectory + "/ssl/bbb-signed")
tr.Processes.Default.ReturnCode = 35

tr = Test.AddTestRun()
tr.StillRunningAfter = ts
tr.StillRunningAfter = server
tr.Processes.Default.Command = (
    "curl -v -k --tls-max 1.2  --cert {1}.pem --key {1}.key --resolve 'bbb.com:{0}:127.0.0.1'" + " https://bbb.com:{0}/xyz").format(
        ts.Variables.ssl_port, Test.TestDirectory + "/ssl/ccc-signed")
tr.Processes.Default.ReturnCode = 35

tr = Test.AddTestRun()
tr.StillRunningAfter = ts
tr.StillRunningAfter = server
tr.Processes.Default.Command = (
    "curl -v -k --tls-max 1.2  --cert {1}.pem --key {1}.key --resolve 'ccc.com:{0}:127.0.0.1'" + " https://ccc.com:{0}/xyz").format(
        ts.Variables.ssl_port, Test.TestDirectory + "/ssl/aaa-signed")
tr.Processes.Default.ReturnCode = 35



'''
Test requiring certificate from user agent
'''
#  Licensed to the Apache Software Foundation (ASF) under one
#  or more contributor license agreements.  See the NOTICE file
#  distributed with this work for additional information
#  regarding copyright ownership.  The ASF licenses this file
#  to you under the Apache License, Version 2.0 (the
#  "License"); you may not use this file except in compliance
#  with the License.  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

Test.Summary = '''
Test various options for requiring certificate from client for mutual authentication TLS
'''

ts = Test.MakeATSProcess("ts", enable_tls=True)
cafile = "{0}/signer.pem".format(Test.RunDirectory)
cafile2 = "{0}/signer2.pem".format(Test.RunDirectory)
server = Test.MakeOriginServer("server")
server2 = Test.MakeOriginServer("server2")

request_header = {"headers": "GET / HTTP/1.1\r\nHost: example.com\r\n\r\n", "timestamp": "1469733493.993", "body": ""}
response_header = {"headers": "HTTP/1.1 200 OK\r\nConnection: close\r\n\r\n", "timestamp": "1469733493.993", "body": ""}
server.addResponse("sessionlog.json", request_header, response_header)
request_header = {"headers": "GET / HTTP/1.1\r\nHost: bar.com\r\n\r\n", "timestamp": "1469733493.993", "body": ""}
response_header = {"headers": "HTTP/1.1 200 OK\r\nConnection: close\r\n\r\n", "timestamp": "1469733493.993", "body": ""}
server.addResponse("sessionlog.json", request_header, response_header)

ts.addSSLfile("ssl/server.pem")
ts.addSSLfile("ssl/server.key")
ts.addSSLfile("ssl/signer.pem")

ts.Disk.records_config.update(
    {
        'proxy.config.ssl.server.cert.path': '{0}'.format(ts.Variables.SSLDir),
        'proxy.config.ssl.server.private_key.path': '{0}'.format(ts.Variables.SSLDir),
        'proxy.config.url_remap.pristine_host_hdr': 1,
        'proxy.config.ssl.client.certification_level': 2,
        'proxy.config.ssl.CA.cert.filename': '{0}/signer.pem'.format(ts.Variables.SSLDir),
        'proxy.config.exec_thread.autoconfig.scale': 1.0,
        'proxy.config.ssl.TLSv1_3.enabled': 0
    })

ts.Disk.ssl_multicert_config.AddLine('dest_ip=* ssl_cert_name=server.pem ssl_key_name=server.key')

# Just map everything through to origin.  This test is concentrating on the user-agent side
ts.Disk.remap_config.AddLine('map / http://127.0.0.1:{0}/'.format(server.Variables.Port))

# Scenario 1:  Default no client cert required.  cert required for bar.com
ts.Disk.sni_yaml.AddLines(
    [
        'sni:',
        '- fqdn: bob.bar.com',
        '  verify_client: NONE',
        '- fqdn: "bob.com"',
        '  verify_client: STRICT',
        '- fqdn: "*.foo.com"',
        '  verify_client: NONE',
        '- fqdn: "*.bar.com"',
        '  verify_client: STRICT',
    ])

ts.Disk.logging_yaml.AddLines(
    '''
logging:
  formats:
    - name: testformat
      format: '%<pssc> %<pquc> %<pscert> %<cscert>'
  logs:
    - mode: ascii
      format: testformat
      filename: squid
'''.split("\n"))

# to foo.com w/o client cert.  Should fail
tr = Test.AddTestRun("Connect to foo.com without cert")
tr.Processes.Default.StartBefore(Test.Processes.ts)
tr.Processes.Default.StartBefore(server)
tr.StillRunningAfter = ts
tr.StillRunningAfter = server
tr.Processes.Default.Command = "curl --tls-max 1.2 -k --resolve 'foo.com:{0}:127.0.0.1' https://foo.com:{0}/case1".format(
    ts.Variables.ssl_port)
tr.Processes.Default.ReturnCode = 35

tr = Test.AddTestRun("Connect to foo.com with bad cert")
tr.Setup.Copy("ssl/server.pem")
tr.Setup.Copy("ssl/server.key")
tr.StillRunningAfter = ts
tr.StillRunningAfter = server
tr.Processes.Default.Command = "curl --tls-max 1.2 -k --cert ./server.pem --key ./server.key --resolve 'foo.com:{0}:127.0.0.1' https://foo.com:{0}/case2".format(
    ts.Variables.ssl_port)
# Should fail with badly signed certs
tr.Processes.Default.ReturnCode = 35

tr = Test.AddTestRun("Connect to foo.com with cert")
tr.Setup.Copy("ssl/signed-foo.pem")
tr.Setup.Copy("ssl/signed-foo.key")
tr.StillRunningAfter = ts
tr.StillRunningAfter = server
tr.Processes.Default.Command = "curl --tls-max 1.2 -k --cert ./signed-foo.pem --key ./signed-foo.key --resolve 'foo.com:{0}:127.0.0.1' https://foo.com:{0}/case3".format(
    ts.Variables.ssl_port)
tr.Processes.Default.ReturnCode = 0
tr.Processes.Default.Streams.All = Testers.ExcludesExpression("error", "Check response")

tr = Test.AddTestRun("Connect to bob.bar.com without cert")
tr.StillRunningAfter = ts
tr.StillRunningAfter = server
tr.Processes.Default.Command = "curl --tls-max 1.2 -k --resolve 'bob.bar.com:{0}:127.0.0.1' https://bob.bar.com:{0}/case4".format(
    ts.Variables.ssl_port)
tr.Processes.Default.ReturnCode = 0
tr.Processes.Default.Streams.All = Testers.ExcludesExpression("alert", "TLS handshake should succeed")

tr = Test.AddTestRun("Connect to bob.bar.com with cert")
tr.Setup.Copy("ssl/signed-bob-bar.pem")
tr.Setup.Copy("ssl/signed-bar.key")
tr.StillRunningAfter = ts
tr.StillRunningAfter = server
tr.Processes.Default.Command = "curl --tls-max 1.2 -k --cert ./signed-bob-bar.pem --key ./signed-bar.key --resolve 'bob.bar.com:{0}:127.0.0.1' https://bob.bar.com:{0}/case5".format(
    ts.Variables.ssl_port)
tr.Processes.Default.ReturnCode = 0
tr.Processes.Default.Streams.All = Testers.ExcludesExpression("error", "Check response")

tr = Test.AddTestRun("Connect to bob.bar.com with bad cert")
tr.Setup.Copy("ssl/server.pem")
tr.Setup.Copy("ssl/server.key")
tr.StillRunningAfter = ts
tr.StillRunningAfter = server
tr.Processes.Default.Command = "curl --tls-max 1.2 -k --cert ./server.pem --key ./server.key --resolve 'bob.bar.com:{0}:127.0.0.1' https://bob.bar.com:{0}/case6".format(
    ts.Variables.ssl_port)
tr.Processes.Default.ReturnCode = 0
tr.Processes.Default.Streams.All = Testers.ExcludesExpression("error", "Check response")

tr = Test.AddTestRun("Connect to bob.foo.com without cert")
tr.StillRunningAfter = ts
tr.StillRunningAfter = server
tr.Processes.Default.Command = "curl --tls-max 1.2 -k --resolve 'bob.foo.com:{0}:127.0.0.1' https://bob.foo.com:{0}/case7".format(
    ts.Variables.ssl_port)
tr.Processes.Default.ReturnCode = 0
tr.Processes.Default.Streams.All = Testers.ExcludesExpression("alert", "TLS handshake should succeed")

tr = Test.AddTestRun("Connect to bob.foo.com with cert")
tr.Setup.Copy("ssl/signed-bob-foo.pem")
tr.Setup.Copy("ssl/signed-foo.key")
tr.StillRunningAfter = ts
tr.StillRunningAfter = server
tr.Processes.Default.Command = "curl --tls-max 1.2 -k --cert ./signed-bob-foo.pem --key ./signed-foo.key --resolve 'bob.foo.com:{0}:127.0.0.1' https://bob.foo.com:{0}/case8".format(
    ts.Variables.ssl_port)
tr.Processes.Default.ReturnCode = 0
tr.Processes.Default.Streams.All = Testers.ExcludesExpression("error", "Check response")

tr = Test.AddTestRun("Connect to bob.foo.com with bad cert")
tr.Setup.Copy("ssl/server.pem")
tr.Setup.Copy("ssl/server.key")
tr.StillRunningAfter = ts
tr.StillRunningAfter = server
tr.Processes.Default.Command = "curl --tls-max 1.2 -k --cert ./server.pem --key ./server.key --resolve 'bob.foo.com:{0}:127.0.0.1' https://bob.foo.com:{0}/case9".format(
    ts.Variables.ssl_port)
tr.Processes.Default.ReturnCode = 0
tr.Processes.Default.Streams.All = Testers.ExcludesExpression("error", "Check response")

tr = Test.AddTestRun("Connect to bar.com without cert")
tr.StillRunningAfter = ts
tr.StillRunningAfter = server
tr.Processes.Default.Command = "curl --tls-max 1.2 -k --resolve 'bar.com:{0}:127.0.0.1' https://bar.com:{0}/case10".format(
    ts.Variables.ssl_port)
tr.Processes.Default.ReturnCode = 35

tr = Test.AddTestRun("Connect to bar.com with cert")
tr.Setup.Copy("ssl/signed-bar.pem")
tr.Setup.Copy("ssl/signed-bar.key")
tr.StillRunningAfter = ts
tr.StillRunningAfter = server
tr.Processes.Default.Command = "curl --tls-max 1.2 -k --cert ./signed-bar.pem --key ./signed-bar.key --resolve 'bar.com:{0}:127.0.0.1' https://bar.com:{0}/case11".format(
    ts.Variables.ssl_port)
tr.Processes.Default.ReturnCode = 0
tr.Processes.Default.Streams.All = Testers.ExcludesExpression("error", "TLS handshake should succeed")

tr = Test.AddTestRun("Connect to bar.com with bad cert")
tr.Setup.Copy("ssl/server.pem")
tr.Setup.Copy("ssl/server.key")
tr.StillRunningAfter = ts
tr.StillRunningAfter = server
tr.Processes.Default.Command = "curl --tls-max 1.2 -k --cert ./server.pem --key ./server.key --resolve 'bar.com:{0}:127.0.0.1' https://bar.com:{0}/case12".format(
    ts.Variables.ssl_port)
tr.Processes.Default.ReturnCode = 35

# Test that the fqdn's match completely.  bob.com should require client certificate. bob.com.com should not
tr = Test.AddTestRun("Connect to bob.com without cert, should fail")
tr.StillRunningAfter = ts
tr.StillRunningAfter = server
tr.Processes.Default.Command = "curl --tls-max 1.2 -k --resolve 'bob.com:{0}:127.0.0.1' https://bob.com:{0}/case13".format(
    ts.Variables.ssl_port)
tr.Processes.Default.ReturnCode = 35

tr = Test.AddTestRun("Connect to bob.foo.com without cert, should succeed")
tr.StillRunningAfter = ts
tr.StillRunningAfter = server
tr.Processes.Default.Command = "curl --tls-max 1.2 -k --resolve 'bob.foo.com:{0}:127.0.0.1' https://bob.foo.com:{0}/case14".format(
    ts.Variables.ssl_port)
tr.Processes.Default.ReturnCode = 0

tr = Test.AddTestRun("Wait for the access log to write out")
tr.Processes.Default.StartBefore(server2, ready=When.FileExists(ts.Disk.squid_log))
tr.StillRunningAfter = ts
tr.StillRunningAfter = server
tr.StillRunningAfter = server2
tr.Processes.Default.Command = 'echo "Log file exists"'
tr.Processes.Default.ReturnCode = 0

ts.Disk.squid_log.Content = "gold/clientcert-accesslog.gold"



'''
'''
#  Licensed to the Apache Software Foundation (ASF) under one
#  or more contributor license agreements.  See the NOTICE file
#  distributed with this work for additional information
#  regarding copyright ownership.  The ASF licenses this file
#  to you under the Apache License, Version 2.0 (the
#  "License"); you may not use this file except in compliance
#  with the License.  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

from enum import Enum

Test.Summary = 'Exercise POST request with max_requests_in'
Test.ContinueOnFail = True


class PostAndMaxRequestsInTest:
    """
    Cover #8273 - Make sure inbound side inactive timeout doesn't happens during outbound side TLS handshake
    """

    def __init__(self):
        self.__setupOriginServer()
        self.__setupTS()

    def __setupOriginServer(self):
        Test.GetTcpPort("server_port")
        self.origin_server = Test.Processes.Process(
            "server", "bash -c '" + Test.TestDirectory + "/server.sh {}'".format(Test.Variables.server_port))

    def __setupTS(self):
        self.ts = Test.MakeATSProcess("ts")

        self.ts.Disk.records_config.update(
            {
                "proxy.config.http.server_ports": f"{self.ts.Variables.port}",
                "proxy.config.net.max_requests_in": 1000,
                'proxy.config.http.connect_attempts_timeout': 1,
                "proxy.config.diags.debug.enabled": 1,
                "proxy.config.diags.debug.tags": "http|socket|v_net_queue",
            })

        self.ts.Disk.remap_config.AddLines([
            f"map / https://127.0.0.1:{Test.Variables.server_port}/",
        ])

    def __testCase0(self):
        """
        - POST request
        - Outbound side TLS Handshake hits connect_attempts_timeout
        - Client gets 502
        """
        tr = Test.AddTestRun()
        tr.Processes.Default.StartBefore(self.origin_server)
        tr.Processes.Default.StartBefore(self.ts)
        tr.Processes.Default.Command = f"curl -X POST --http1.1 -vs http://127.0.0.1:{self.ts.Variables.port}/ --data key=value"
        tr.Processes.Default.ReturnCode = 0
        tr.Processes.Default.Streams.stdout = "gold/post_slow_server_max_requests_in_0_stdout.gold"
        tr.Processes.Default.Streams.stderr = "gold/post_slow_server_max_requests_in_0_stderr.gold"
        tr.StillRunningAfter = self.ts

    def run(self):
        self.__testCase0()


PostAndMaxRequestsInTest().run()



#  Licensed to the Apache Software Foundation (ASF) under one
#  or more contributor license agreements.  See the NOTICE file
#  distributed with this work for additional information
#  regarding copyright ownership.  The ASF licenses this file
#  to you under the Apache License, Version 2.0 (the
#  "License"); you may not use this file except in compliance
#  with the License.  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

from typing import Optional
import re

Test.Summary = 'Verify http2.no_activity_timeout_(in|out)'


class Test_http2_no_activity_timeout:
    """Configure a test for http2 no_activity_timeout_(in|out)."""

    replay_file = "replay/http2_no_activity_timeout.replay.yaml"
    client_counter: int = 0
    ts_counter: int = 0
    server_counter: int = 0

    def __init__(
            self,
            name: str,
            replay_keys: Optional[str] = None,
            no_activity_timeout_in: Optional[int] = None,
            expect_in_timeout=False,
            no_activity_timeout_out: Optional[int] = None,
            expect_out_timeout=False):
        """Initialize the test.

        :param name: The name of the test.
        :param replay_keys: The set of keys to replay from the replay file. If
          not provided, all keys will be replayed.
        :param no_activity_timeout_in: Configuration value for
          ts.http2.no_activity_timeout_in. None means that the default value will
          be used.
        :param expect_in_timeout: True if there is an expected inbound timeout.
        :param no_activity_timeout_out: Configuration value for
          ts.http2.no_activity_timeout_out. None means that the default value will
          be used.
        :param expect_out_timeout: True if there is an expected outbound timeout.
        """
        self._name = name
        self._replay_keys = replay_keys
        self._no_activity_timeout_in = no_activity_timeout_in
        self._expect_in_timeout = expect_in_timeout
        self._no_activity_timeout_out = no_activity_timeout_out
        self._expect_out_timeout = expect_out_timeout

    def _configure_server(self, tr: 'TestRun'):
        """Configure the server.

        :param tr: The TestRun object to associate the server process with.
        """
        self._server = tr.AddVerifierServerProcess(f"server_{Test_http2_no_activity_timeout.server_counter}", self.replay_file)
        Test_http2_no_activity_timeout.server_counter += 1

    def _configure_traffic_server(self, tr: 'TestRun'):
        """Configure Traffic Server.

        :param tr: The TestRun object to associate the ts process with.
        """
        self._ts = tr.MakeATSProcess(f"ts-{Test_http2_no_activity_timeout.ts_counter}", enable_tls=True, enable_cache=False)
        Test_http2_no_activity_timeout.ts_counter += 1

        self._ts.addSSLfile("ssl/cert.crt")
        self._ts.addSSLfile("ssl/private-key.key")
        self._ts.Disk.ssl_multicert_config.AddLine(
            f'dest_ip=* ssl_cert_name={self._ts.Variables.SSLDir}/cert.crt '
            f'ssl_key_name={self._ts.Variables.SSLDir}/private-key.key')

        self._ts.Disk.records_config.update(
            {
                'proxy.config.diags.debug.enabled': 1,
                'proxy.config.diags.debug.tags': 'http|socket|inactivity_cop',
                'proxy.config.ssl.server.cert.path': self._ts.Variables.SSLDir,
                'proxy.config.ssl.server.private_key.path': self._ts.Variables.SSLDir,
                "proxy.config.ssl.client.verify.server.policy": 'PERMISSIVE',
                'proxy.config.ssl.client.alpn_protocols': 'h2,http/1.1',
            })

        if self._no_activity_timeout_in is not None:
            self._ts.Disk.records_config.update({
                'proxy.config.http2.no_activity_timeout_in': self._no_activity_timeout_in,
            })
        if self._no_activity_timeout_out is not None:
            self._ts.Disk.records_config.update({
                'proxy.config.http2.no_activity_timeout_out': self._no_activity_timeout_out,
            })

        self._ts.Disk.remap_config.AddLine(f'map / https://127.0.0.1:{self._server.Variables.https_port}')

    def run(self):
        """Run the test."""
        tr = Test.AddTestRun(self._name)
        self._configure_server(tr)
        self._configure_traffic_server(tr)

        tr.Processes.Default.StartBefore(self._server)
        tr.Processes.Default.StartBefore(self._ts)

        tr.AddVerifierClientProcess(
            f'client-{Test_http2_no_activity_timeout.client_counter}',
            self.replay_file,
            https_ports=[self._ts.Variables.ssl_port],
            keys=self._replay_keys)
        Test_http2_no_activity_timeout.client_counter += 1

        if self._expect_in_timeout:
            tr.Processes.Default.Streams.All += Testers.IncludesExpression(
                "SSL_read error", "The client should have a read error due to an ATS timeout.")
            self._ts.Disk.traffic_out.Content += Testers.IncludesExpression(
                "http2_cs.*Closing event:.*TIMEOUT", "We should detect a client side timeout.")
        elif self._expect_out_timeout:
            # There should be two origin connections:
            # 1. For the first no delay transaction.
            # 2. Another after a timeout, closing the first.
            self._server.Streams.All += Testers.IncludesExpression(
                "Negotiated ALPN from client ALPN.*Negotiated ALPN from client ALPN",
                "A second server side connection should be needed after the first times out.",
                reflags=re.MULTILINE | re.DOTALL)
            self._ts.Disk.traffic_out.Content += Testers.IncludesExpression(
                "http2_cs.*Closing event:.*TIMEOUT", "We should detect a server side timeout.")


test0 = Test_http2_no_activity_timeout("Default no activity timeout", expect_in_timeout=False, expect_out_timeout=False)
test0.run()

test1 = Test_http2_no_activity_timeout(
    "Client (in) side inactivity timeout",
    no_activity_timeout_in=1,
    replay_keys="no_delay 3_second_request_delay",
    expect_in_timeout=True,
    expect_out_timeout=False)
test1.run()

test2 = Test_http2_no_activity_timeout(
    "Server (out) side inactivity timeout",
    no_activity_timeout_out=1,
    replay_keys="no_delay 3_second_request_delay",
    expect_in_timeout=False,
    expect_out_timeout=True)
test2.run()



'''
'''
#  Licensed to the Apache Software Foundation (ASF) under one
#  or more contributor license agreements.  See the NOTICE file
#  distributed with this work for additional information
#  regarding copyright ownership.  The ASF licenses this file
#  to you under the Apache License, Version 2.0 (the
#  "License"); you may not use this file except in compliance
#  with the License.  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

Test.Summary = 'Testing ATS inactivity timeout'

Test.SkipUnless(
    Condition.HasCurlFeature('http2'), Condition.HasProgram("telnet", "Need telnet to shutdown when server shuts down tcp"),
    Condition.HasProgram("nc", "Need nc to send data to server"))

ts = Test.MakeATSProcess("ts", enable_tls=True)

ts.addSSLfile("../tls/ssl/server.pem")
ts.addSSLfile("../tls/ssl/server.key")

ts.Disk.records_config.update(
    {
        'proxy.config.ssl.server.cert.path': '{0}'.format(ts.Variables.SSLDir),
        'proxy.config.ssl.server.private_key.path': '{0}'.format(ts.Variables.SSLDir),
        'proxy.config.http.transaction_no_activity_timeout_in': 6,
        'proxy.config.http.accept_no_activity_timeout': 2,
        'proxy.config.net.default_inactivity_timeout': 10,
        'proxy.config.net.defer_accept': 0  # Must turn off defer accept to test the raw TCP case
    })

ts.Disk.ssl_multicert_config.AddLine('dest_ip=* ssl_cert_name=server.pem ssl_key_name=server.key')

# case 1 TLS with no data
tr = Test.AddTestRun("tr")
tr.Setup.Copy("time_client.sh")
tr.Processes.Default.StartBefore(ts)
tr.Processes.Default.Command = 'sh ./time_client.sh \'openssl s_client -ign_eof -connect 127.0.0.1:{0}\''.format(
    ts.Variables.ssl_port)
tr.Processes.Default.Streams.stdout = Testers.ContainsExpression("Accept timeout", "Request should fail from the accept timeout")

# case 2 TLS with incomplete request header
tr2 = Test.AddTestRun("tr")
tr2.Processes.Default.Command = 'echo "GET /.html HTTP/1.1" | sh ./time_client.sh \'openssl s_client -ign_eof -connect 127.0.0.1:{0}\''.format(
    ts.Variables.ssl_port)
tr2.Processes.Default.Streams.stdout = Testers.ContainsExpression(
    "Transaction inactivity timeout", "Request should fail with transaction inactivity timeout")

# case 3 TCP with no data
tr3 = Test.AddTestRun("tr")
tr3.Processes.Default.Command = 'sh ./time_client.sh \'telnet 127.0.0.1 {0}\''.format(ts.Variables.port)
tr3.Processes.Default.Streams.stdout = Testers.ContainsExpression("Accept timeout", "Request should fail from the accept timeout")

# case 4 TCP with incomplete request header
tr4 = Test.AddTestRun("tr")
tr4.Setup.Copy("create_request.sh")
tr4.Processes.Default.Command = 'sh  ./time_client.sh \'nc -c ./create_request.sh 127.0.0.1 {0}\''.format(ts.Variables.port)
tr4.Processes.Default.Streams.stdout = Testers.ContainsExpression(
    "Transaction inactivity timeout", "Request should fail with transaction inactivity timeout")



'''
'''
#  Licensed to the Apache Software Foundation (ASF) under one
#  or more contributor license agreements.  See the NOTICE file
#  distributed with this work for additional information
#  regarding copyright ownership.  The ASF licenses this file
#  to you under the Apache License, Version 2.0 (the
#  "License"); you may not use this file except in compliance
#  with the License.  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

Test.Summary = '''
Test conf_remap using a yaml file.
'''

Test.ContinueOnFail = True


class conf_remap_yaml_load_test:
    """Test conf_remap using a yaml file."""

    client_counter: int = 0
    ts_counter: int = 0
    server_counter: int = 0

    def __init__(self, name: str, gold_file="", remap_filename="", remap_content=""):
        """Initialize the test.
        :param name: The name of the test.
        :param gold_file: Gold file to be checked.
        :param remap_filename: Remap yaml filename.
        :param remap_content: remap yaml file content.
        """
        self.name = name
        self.gold_file = gold_file
        self._remap_filename = remap_filename
        self._remap_content = remap_content

    def _configure_server(self, tr: 'TestRun'):
        """Configure the server.

        :param tr: The TestRun object to associate the server process with.
        """
        server = Test.MakeOriginServer(f"server-{conf_remap_yaml_load_test.ts_counter}", lookup_key="{%Host}{PATH}")
        request_header2 = {
            "headers": "GET /test HTTP/1.1\r\nHost: www.testexample.com\r\n\r\n",
            "timestamp": "1469733493.993",
            "body": ""
        }
        response_header2 = {"headers": "HTTP/1.1 200 OK\r\nConnection: close\r\n\r\n", "timestamp": "1469733493.993", "body": ""}

        server.addResponse("sessionfile.log", request_header2, response_header2)
        conf_remap_yaml_load_test.server_counter += 1
        self._server = server

    def _configure_traffic_server(self, tr: 'TestRun'):
        """Configure Traffic Server.

        :param tr: The TestRun object to associate the ts process with.
        """
        ts = Test.MakeATSProcess(f"ts-{conf_remap_yaml_load_test.ts_counter}")

        conf_remap_yaml_load_test.ts_counter += 1
        ts.Disk.records_config.update(
            '''
            diags:
                debug:
                    enabled: 1
                    tags: conf_remap
            dns:
                resolv_conf: NULL
            http:
                referer_filter: 1
            url_remap:
                pristine_host_hdr: 0 # make sure is 0

        ''')
        self._ts = ts

    def run(self, diags_fail_exp="", ts_retcode=0):
        """Run the test.
        :param diags_fail_exp: Text to be included to validate the error.
        :param ts_retcode: Expected return code from TS.
        """
        tr = Test.AddTestRun(self.name)
        self._configure_server(tr)
        self._configure_traffic_server(tr)

        tr.Processes.Default.StartBefore(self._server)
        tr.Processes.Default.StartBefore(self._ts)

        self._ts.ReturnCode = ts_retcode

        if ts_retcode > 0:  # we could have errors logged and yet, we still want to move on.
            self._ts.Ready = 0

        if diags_fail_exp != "":
            # some error logs will be written to the diags.
            self._ts.Disk.diags_log.Content = Testers.IncludesExpression(diags_fail_exp, "Have a look.")
        else:
            tr.Processes.Default.ReturnCode = 0

        if self.gold_file:
            tr.Processes.Default.Streams.stderr = self.gold_file

        if self._remap_filename != "" and self._remap_content != "":
            self._ts.Disk.MakeConfigFile(self._remap_filename).update(self._remap_content)
            self._ts.Disk.remap_config.AddLine(
                f'map http://www.testexample.com/ http://127.0.0.1:{self._server.Variables.Port} @plugin=conf_remap.so @pparam={self._remap_filename}'
            )

        tr.Processes.Default.Command = 'curl --proxy 127.0.0.1:{0} "http://www.testexample.com/test" -H "Host: www.testexample.com" --verbose'.format(
            self._ts.Variables.port)
        conf_remap_yaml_load_test.client_counter += 1


test0 = conf_remap_yaml_load_test(
    "Test success",
    gold_file="gold/200OK_test.gold",
    remap_filename="testexample_remap.yaml",
    remap_content='''
    records:
      url_remap:
        pristine_host_hdr: 1
    ''')
test0.run()

test1 = conf_remap_yaml_load_test(
    "Test mismatch type",
    remap_filename="mismatch_field_type_remap.yaml",
    remap_content='''
    records:
      url_remap:
        pristine_host_hdr: !!float '1'
    ''')
test1.run(diags_fail_exp="'proxy.config.url_remap.pristine_host_hdr' variable type mismatch", ts_retcode=33)

test2 = conf_remap_yaml_load_test(
    "Test invalid variable",
    remap_filename="invalid1_field_type_remap.yaml",
    remap_content='''
    records:
      plugin:
        dynamic_reload_mode: 1
    ''')

test2.run(
    diags_fail_exp="'proxy.config.plugin.dynamic_reload_mode' is not a configuration variable or cannot be overridden",
    ts_retcode=33)

# We let the conf_remap parse two fields, only one is valid, we expect ATS to start and the invalid fields ignored.
test3 = conf_remap_yaml_load_test(
    "Test success",
    gold_file="gold/200OK_test.gold",
    remap_filename="testexample2_remap.yaml",
    remap_content='''
    records:
      plugin:
        dynamic_reload_mode: 1

      url_remap:
        pristine_host_hdr: 1
    ''')
test3.run(diags_fail_exp="'proxy.config.plugin.dynamic_reload_mode' is not a configuration variable or cannot be overridden")

# Check null values
test4 = conf_remap_yaml_load_test(
    "Test success - with NULL variable",
    gold_file="gold/200OK_test.gold",
    remap_filename="testexample_remap.yaml",
    remap_content='''
    records:
      url_remap:
        pristine_host_hdr: 1
      hostdb:
        ip_resolve: "NULL" # We want to make sure this gets read as it should. "NULL" could be the value of this field.
    ''')
test4.run()



'''
'''
#  Licensed to the Apache Software Foundation (ASF) under one
#  or more contributor license agreements.  See the NOTICE file
#  distributed with this work for additional information
#  regarding copyright ownership.  The ASF licenses this file
#  to you under the Apache License, Version 2.0 (the
#  "License"); you may not use this file except in compliance
#  with the License.  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

import os

Test.Summary = '''
Test the Multiplexer plugin.
'''

Test.SkipUnless(Condition.PluginExists('multiplexer.so'))


class MultiplexerTestBase:
    """
    Encapsulates the base configuration used by each test.
    """

    client_counter = 0
    dns_counter = 0
    server_counter = 0
    ts_counter = 0

    def __init__(self, replay_file, multiplexed_host_replay_file, skip_post):
        self.replay_file = replay_file
        self.multiplexed_host_replay_file = multiplexed_host_replay_file

        self.setupServers()
        self.setupDns()
        self.setupTS(skip_post)

    def setupDns(self):
        counter = MultiplexerTestBase.dns_counter
        MultiplexerTestBase.dns_counter += 1
        self.dns = Test.MakeDNServer(f"dns_{counter}", default='127.0.0.1')

    def setupServers(self):
        counter = MultiplexerTestBase.server_counter
        MultiplexerTestBase.server_counter += 1
        self.server_origin = Test.MakeVerifierServerProcess(f"server_origin_{counter}", self.replay_file)
        self.server_http = Test.MakeVerifierServerProcess(f"server_http_{counter}", self.multiplexed_host_replay_file)
        self.server_https = Test.MakeVerifierServerProcess(f"server_https_{counter}", self.multiplexed_host_replay_file)

        # The origin should never receive "X-Multiplexer: copy"
        self.server_origin.Streams.All += Testers.ExcludesExpression(
            'X-Multiplexer: copy', 'Verify the original server target never receives a "copy".')

        # Nor should the multiplexed hosts receive an "original" X-Multiplexer value.
        self.server_http.Streams.All += Testers.ExcludesExpression(
            'X-Multiplexer: original', 'Verify the HTTP multiplexed host does not receive an "original".')
        self.server_https.Streams.All += Testers.ExcludesExpression(
            'X-Multiplexer: original', 'Verify the HTTPS multiplexed host does not receive an "original".')
        self.server_https.Streams.All += Testers.ExcludesExpression(r'\[ERROR\]', 'Verify there were no errors in the replay.')

        # In addition, the original server should always receive the POST and
        # PUT requests.
        self.server_origin.Streams.All += Testers.ContainsExpression(
            'uuid: POST', "Verify the client's original target received the POST transaction.")
        self.server_origin.Streams.All += Testers.ContainsExpression(
            'uuid: PUT', "Verify the client's original target received the PUT transaction.")
        self.server_origin.Streams.All += Testers.ExcludesExpression(r'\[ERROR\]', 'Verify there were no errors in the replay.')
        # The chunked POST should go to the origin.
        self.server_origin.Streams.All += Testers.ContainsExpression(
            'uuid: CHUNKED_POST', "Verify the client's original target received the chunked POST transaction.")

        # Under all configurations, the GET request should be multiplexed.
        self.server_origin.Streams.All += Testers.ContainsExpression(
            'X-Multiplexer: original', 'Verify the client\'s original target received the "original" request.')
        self.server_origin.Streams.All += Testers.ContainsExpression(
            'uuid: GET', "Verify the client's original target received the GET request.")

        self.server_http.Streams.All += Testers.ContainsExpression(
            'X-Multiplexer: copy', 'Verify the HTTP server received a "copy" of the request.')
        self.server_http.Streams.All += Testers.ContainsExpression('uuid: GET', "Verify the HTTP server received the GET request.")
        self.server_http.Streams.All += Testers.ExcludesExpression(r'\[ERROR\]', 'Verify there were no errors in the replay.')
        # Chunked POST requests are not supported for multiplexing.
        self.server_http.Streams.All += Testers.ExcludesExpression(
            'uuid: CHUNKED_POST', 'We do not expect a multiplexed chunked POST.')

        self.server_https.Streams.All += Testers.ContainsExpression(
            'X-Multiplexer: copy', 'Verify the HTTPS server received a "copy" of the request.')
        self.server_https.Streams.All += Testers.ContainsExpression(
            'uuid: GET', "Verify the HTTPS server received the GET request.")
        # Chunked POST requests are not supported for multiplexing.
        self.server_https.Streams.All += Testers.ExcludesExpression(
            'uuid: CHUNKED_POST', 'We do not expect a multiplexed chunked POST.')

        # Verify that the HTTPS server receives a TLS connection.
        self.server_https.Streams.All += Testers.ContainsExpression(
            'Finished accept using TLSSession', "Verify the HTTPS was indeed used by the HTTPS server.")

    def setupTS(self, skip_post):
        counter = MultiplexerTestBase.ts_counter
        MultiplexerTestBase.ts_counter += 1
        self.ts = Test.MakeATSProcess(f"ts_{counter}", enable_tls=True, enable_cache=False)
        self.ts.addDefaultSSLFiles()
        self.ts.Disk.records_config.update(
            {
                "proxy.config.ssl.server.cert.path": f'{self.ts.Variables.SSLDir}',
                "proxy.config.ssl.server.private_key.path": f'{self.ts.Variables.SSLDir}',
                "proxy.config.ssl.client.verify.server.policy": 'PERMISSIVE',
                'proxy.config.diags.debug.enabled': 1,
                'proxy.config.diags.debug.tags': 'http|multiplexer',
                'proxy.config.dns.nameservers': f'127.0.0.1:{self.dns.Variables.Port}',
                'proxy.config.dns.resolv_conf': 'NULL',
            })
        self.ts.Disk.ssl_multicert_config.AddLine('dest_ip=* ssl_cert_name=server.pem ssl_key_name=server.key')
        skip_remap_param = ''
        if skip_post:
            skip_remap_param = ' @pparam=proxy.config.multiplexer.skip_post_put=1'
        self.ts.Disk.remap_config.AddLines(
            [
                f'map https://origin.server.com https://backend.origin.server.com:{self.server_origin.Variables.https_port} '
                f'@plugin=multiplexer.so @pparam=nontls.server.com @pparam=tls.server.com'
                f'{skip_remap_param}',

                # Now create remap entries for the multiplexed hosts: one that
                # verifies HTTP, and another that verifies HTTPS.
                f'map http://nontls.server.com http://backend.nontls.server.com:{self.server_http.Variables.http_port}',
                f'map http://tls.server.com https://backend.tls.server.com:{self.server_https.Variables.https_port}',
            ])

    def run(self):
        tr = Test.AddTestRun()
        self.ts.StartBefore(self.dns)
        tr.Processes.Default.StartBefore(self.server_origin)
        tr.Processes.Default.StartBefore(self.server_http)
        tr.Processes.Default.StartBefore(self.server_https)
        tr.Processes.Default.StartBefore(self.ts)

        counter = MultiplexerTestBase.client_counter
        MultiplexerTestBase.client_counter += 1
        client = tr.AddVerifierClientProcess(f"client_{counter}", self.replay_file, https_ports=[self.ts.Variables.ssl_port])
        client.Streams.All += Testers.ExcludesExpression(r'\[ERROR\]', 'Verify there were no errors in the replay.')


class MultiplexerTest(MultiplexerTestBase):
    """
    Exercise multiplexing without skip_post configuration.
    """

    replay_file = os.path.join("replays", "multiplexer_original.replay.yaml")
    multiplexed_host_replay_file = os.path.join("replays", "multiplexer_copy.replay.yaml")

    def __init__(self):
        super().__init__(MultiplexerTest.replay_file, MultiplexerTest.multiplexed_host_replay_file, skip_post=False)

    def setupServers(self):
        super().setupServers()

        # Both of the multiplexed hosts should receive the POST because skip_post
        # is disabled.
        self.server_http.Streams.All += Testers.ContainsExpression(
            'uuid: POST', "Verify the HTTP server received the POST request.")
        self.server_https.Streams.All += Testers.ContainsExpression(
            'uuid: POST', "Verify the HTTPS server received the POST request.")
        self.server_http.Streams.All += Testers.ContainsExpression(
            'x-response: second', "Verify the HTTP server sent the POST response.")
        self.server_https.Streams.All += Testers.ContainsExpression(
            'x-response: second', "Verify the HTTPS server sent the POST response.")

        # Same with PUT
        self.server_http.Streams.All += Testers.ContainsExpression('uuid: PUT', "Verify the HTTP server received the PUT request.")
        self.server_https.Streams.All += Testers.ContainsExpression(
            'uuid: PUT', "Verify the HTTPS server received the PUT request.")


class MultiplexerSkipPostTest(MultiplexerTestBase):
    """
    Exercise multiplexing with skip_post configuration.
    """

    replay_file = os.path.join("replays", "multiplexer_original_skip_post.replay.yaml")
    multiplexed_host_replay_file = os.path.join("replays", "multiplexer_copy_skip_post.replay.yaml")

    def __init__(self):
        super().__init__(MultiplexerSkipPostTest.replay_file, MultiplexerSkipPostTest.multiplexed_host_replay_file, skip_post=True)

    def setupServers(self):
        super().setupServers()

        # Neither of the multiplexed hosts should receive the POST because skip_post
        # is enabled.
        self.server_http.Streams.All += Testers.ExcludesExpression(
            'uuid: POST', "Verify the HTTP server did not receive the POST request.")
        self.server_https.Streams.All += Testers.ExcludesExpression(
            'uuid: POST', "Verify the HTTPS server did not receive the POST request.")

        # Same with PUT.
        self.server_http.Streams.All += Testers.ExcludesExpression(
            'uuid: PUT', "Verify the HTTP server did not receive the PUT request.")
        self.server_https.Streams.All += Testers.ExcludesExpression(
            'uuid: PUT', "Verify the HTTPS server did not receive the PUT request.")


MultiplexerTest().run()
MultiplexerSkipPostTest().run()



"""Verify ATS handles a server that replies before receiving a full request."""

#  Licensed to the Apache Software Foundation (ASF) under one
#  or more contributor license agreements.  See the NOTICE file
#  distributed with this work for additional information
#  regarding copyright ownership.  The ASF licenses this file
#  to you under the Apache License, Version 2.0 (the
#  "License"); you may not use this file except in compliance
#  with the License.  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

from ports import get_port
import sys

Test.Summary = __doc__


class QuickServerTest:
    """Verify that ATS doesn't delay responses behind slow posts."""

    _init_file = '__init__.py'
    _http_utils = 'http_utils.py'
    _slow_post_client = 'slow_post_client.py'
    _quick_server = 'quick_server.py'

    _dns_counter = 0
    _server_counter = 0
    _ts_counter = 0

    def __init__(self, abort_request: bool, drain_request: bool, abort_response_headers: bool):
        """Initialize the test.

        :param drain_request: Whether the server should drain the request body.
        :param abort_request: Whether the client should abort the request body.
        before disconnecting.
        """
        self._should_drain_request = drain_request
        self._should_abort_request = abort_request
        self._should_abort_response_headers = abort_response_headers

    def _configure_dns(self, tr: 'TestRun') -> None:
        """Configure the DNS.

        :param tr: The test run to associate with the DNS process with.
        """
        self._dns = tr.MakeDNServer(f'dns-{QuickServerTest._dns_counter}', default='127.0.0.1')
        QuickServerTest._dns_counter += 1

    def _configure_server(self, tr: 'TestRun'):
        """Configure the origin server.

        This server replies with a response immediately after receiving the
        request headers.

        :param tr: The test run to associate with the server process with.
        """
        server = tr.Processes.Process(f'server-{QuickServerTest._server_counter}')
        QuickServerTest._server_counter += 1
        port = get_port(server, "http_port")
        server.Command = \
            f'{sys.executable} {self._quick_server} 127.0.0.1 {port} '
        if self._should_drain_request:
            server.Command += '--drain-request '
        if self._should_abort_response_headers:
            server.Command += '--abort-response-headers '
        server.Ready = When.PortOpenv4(port)
        self._server = server

    def _configure_traffic_server(self, tr: 'TestRun'):
        """Configure ATS.

        :param tr: The test run to associate with the ATS process with.
        """
        self._ts = tr.MakeATSProcess(f'ts-{QuickServerTest._ts_counter}')
        QuickServerTest._ts_counter += 1
        self._ts.Disk.remap_config.AddLine(f'map / http://quick.server.com:{self._server.Variables.http_port}')
        self._ts.Disk.records_config.update(
            {
                'proxy.config.diags.debug.enabled': 1,
                'proxy.config.diags.debug.tags': 'http|dns|hostdb',
                'proxy.config.dns.nameservers': f'127.0.0.1:{self._dns.Variables.Port}',
                'proxy.config.dns.resolv_conf': 'NULL',
            })

    def run(self):
        """Run the test."""
        tr = Test.AddTestRun(
            f'Aborting request: {self._should_abort_request}, '
            f'Draining request: {self._should_drain_request}, '
            f'Aborting response headers: {self._should_abort_response_headers}')

        self._configure_dns(tr)
        self._configure_server(tr)
        self._configure_traffic_server(tr)

        tr.Setup.CopyAs(self._init_file, Test.RunDirectory)
        tr.Setup.CopyAs(self._http_utils, Test.RunDirectory)
        tr.Setup.CopyAs(self._slow_post_client, Test.RunDirectory)
        tr.Setup.CopyAs(self._quick_server, Test.RunDirectory)

        client_command = (f'{sys.executable} {self._slow_post_client} '
                          '127.0.0.1 '
                          f'{self._ts.Variables.port} ')
        if not self._should_abort_request:
            client_command += '--finish-request '
        p = tr.Processes.Default
        p.Command = client_command
        if self._should_abort_request or self._should_abort_response_headers:
            p.Streams.All += Testers.ExcludesExpression('HTTP/1.1 200 OK', 'Verify response was received')
        else:
            p.Streams.All += Testers.ContainsExpression('HTTP/1.1 200 OK', 'Verify response was received')

        p.ReturnCode = 0
        self._ts.StartBefore(self._dns)
        self._ts.StartBefore(self._server)
        p.StartBefore(self._ts)
        tr.Timeout = 10


for abort_request in [True, False]:
    for drain_request in [True, False]:
        for abort_response_headers in [True, False]:
            test = QuickServerTest(abort_request, drain_request, abort_response_headers)
            test.run()



'''
'''
#  Licensed to the Apache Software Foundation (ASF) under one
#  or more contributor license agreements.  See the NOTICE file
#  distributed with this work for additional information
#  regarding right ownership.  The ASF licenses this file
#  to you under the Apache License, Version 2.0 (the
#  "License"); you may not use this file except in compliance
#  with the License.  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

Test.Summary = '''
Test that Trafficserver starts with default configurations.
'''

ts = Test.MakeATSProcess("ts", enable_tls=True)
server = Test.MakeOriginServer("server")

ts.addSSLfile("ssl/passphrase.pem")
ts.addSSLfile("ssl/passphrase.key")

ts.addSSLfile("ssl/passphrase2.pem")
ts.addSSLfile("ssl/passphrase2.key")

ts.Disk.remap_config.AddLine(f"map https://passphrase:{ts.Variables.ssl_port}/ http://127.0.0.1:{server.Variables.Port}")
ts.Disk.records_config.update(
    {
        'proxy.config.diags.debug.enabled': 1,
        'proxy.config.diags.debug.tags': 'ssl_load|http',
        'proxy.config.ssl.server.cert.path': '{0}'.format(ts.Variables.SSLDir),
        'proxy.config.ssl.server.private_key.path': '{0}'.format(ts.Variables.SSLDir),
    })

ts.Disk.ssl_multicert_config.AddLines(
    [
        'dest_ip=* ssl_cert_name=passphrase.pem ssl_key_name=passphrase.key ssl_key_dialog="exec:/bin/bash -c \'echo -n passphrase\'"',
    ])

request_header = {"headers": "GET / HTTP/1.1\r\nHost: bogus\r\n\r\n", "timestamp": "1469733493.993", "body": ""}
response_header = {"headers": "HTTP/1.1 200 OK\r\nConnection: close\r\n\r\n", "timestamp": "1469733493.993", "body": "success!"}
server.addResponse("sessionlog.json", request_header, response_header)

tr = Test.AddTestRun("use a key with passphrase")
tr.Setup.Copy("ssl/signer.pem")
tr.Processes.Default.Command = f"curl -v --cacert ./signer.pem  --resolve 'passphrase:{ts.Variables.ssl_port}:127.0.0.1' https://passphrase:{ts.Variables.ssl_port}/"
tr.ReturnCode = 0
tr.Processes.Default.StartBefore(server)
tr.Processes.Default.StartBefore(Test.Processes.ts)
tr.Processes.Default.Streams.stderr.Content = Testers.ContainsExpression("200", "expected 200 OK response")
tr.Processes.Default.Streams.stdout.Content = Testers.ContainsExpression("success!", "expected success")
tr.StillRunningAfter = server
tr.StillRunningAfter = ts

tr2 = Test.AddTestRun("Update config files")
# Update the multicert config
sslcertpath = ts.Disk.ssl_multicert_config.AbsPath
tr2.Disk.File(sslcertpath, id="ssl_multicert_config", typename="ats:config"),
tr2.Disk.ssl_multicert_config.AddLines(
    [
        'dest_ip=* ssl_cert_name=passphrase2.pem ssl_key_name=passphrase2.key ssl_key_dialog="exec:/bin/bash -c \'echo -n passphrase\'"',
    ])
tr2.StillRunningAfter = ts
tr2.StillRunningAfter = server
tr2.Processes.Default.Command = 'echo Updated configs'
# Need to copy over the environment so traffic_ctl knows where to find the unix domain socket
tr2.Processes.Default.Env = ts.Env
tr2.Processes.Default.ReturnCode = 0

tr2reload = Test.AddTestRun("Reload config")
tr2reload.StillRunningAfter = ts
tr2reload.StillRunningAfter = server
tr2reload.Processes.Default.Command = 'traffic_ctl config reload'
# Need to copy over the environment so traffic_ctl knows where to find the unix domain socket
tr2reload.Processes.Default.Env = ts.Env
tr2reload.Processes.Default.ReturnCode = 0

tr3 = Test.AddTestRun("use a key with passphrase")
tr3.Setup.Copy("ssl/signer.pem")
tr3.Processes.Default.Command = f"curl -v --cacert ./signer.pem  --resolve 'passphrase:{ts.Variables.ssl_port}:127.0.0.1' https://passphrase:{ts.Variables.ssl_port}/"
tr3.ReturnCode = 0
tr3.Processes.Default.Streams.stderr.Content = Testers.ContainsExpression("200", "expected 200 OK response")
tr3.Processes.Default.Streams.stdout.Content = Testers.ContainsExpression("success!", "expected success")
tr3.StillRunningAfter = server
tr3.StillRunningAfter = ts



'''
'''
#  Licensed to the Apache Software Foundation (ASF) under one
#  or more contributor license agreements.  See the NOTICE file
#  distributed with this work for additional information
#  regarding copyright ownership.  The ASF licenses this file
#  to you under the Apache License, Version 2.0 (the
#  "License"); you may not use this file except in compliance
#  with the License.  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

import os
import re

Test.Summary = '''
Test ATS offering both RSA and EC certificates
Combined key and cert files.  Faulty key path
'''

# Define default ATS
ts = Test.MakeATSProcess("ts", enable_tls=True)
server = Test.MakeOriginServer("server", ssl=True)
dns = Test.MakeDNServer("dns")

request_header = {"headers": "GET / HTTP/1.1\r\n\r\n", "timestamp": "1469733493.993", "body": ""}
response_header = {"headers": "HTTP/1.1 200 OK\r\nConnection: close\r\n\r\n", "timestamp": "1469733493.993", "body": ""}
server.addResponse("sessionlog.json", request_header, response_header)

# add ssl materials like key, certificates for the server
ts.addSSLfile("ssl/signed-foo.pem")
ts.addSSLfile("ssl/signed-foo.key")
ts.addSSLfile("ssl/signed-foo-ec.pem")
ts.addSSLfile("ssl/signed-foo-ec.key")
ts.addSSLfile("ssl/signed-san.pem")
ts.addSSLfile("ssl/signed-san.key")
ts.addSSLfile("ssl/signed-san-ec.pem")
ts.addSSLfile("ssl/signed-san-ec.key")
ts.addSSLfile("ssl/combined-ec.pem")
ts.addSSLfile("ssl/combined.pem")
ts.addSSLfile("ssl/signer.pem")
ts.addSSLfile("ssl/signer.key")

ts.Disk.remap_config.AddLine('map / https://foo.com:{1}'.format(ts.Variables.ssl_port, server.Variables.SSL_Port))

ts.Disk.ssl_multicert_config.AddLines(
    [
        'ssl_cert_name=combined-ec.pem,combined.pem',
        'ssl_cert_name=signed-foo-ec.pem,signed-foo.pem',
        'dest_ip=* ssl_cert_name=signed-san-ec.pem,signed-san.pem',
    ])

# Case 1, global config policy=permissive properties=signature
#         override for foo.com policy=enforced properties=all
ts.Disk.records_config.update(
    {
        'proxy.config.ssl.server.cert.path': '{0}'.format(ts.Variables.SSLDir),
        'proxy.config.ssl.server.private_key.path': '/tmp',  # Faulty key path should not matter, since there are no key files
        'proxy.config.ssl.server.cipher_suite': 'ECDHE-ECDSA-AES128-GCM-SHA256:ECDHE-RSA-AES128-GCM-SHA256',
        'proxy.config.url_remap.pristine_host_hdr': 1,
        'proxy.config.dns.nameservers': '127.0.0.1:{0}'.format(dns.Variables.Port),
        'proxy.config.exec_thread.autoconfig.scale': 1.0,
        'proxy.config.dns.resolv_conf': 'NULL',
        'proxy.config.diags.debug.tags': 'ssl',
        'proxy.config.diags.debug.enabled': 0
    })

dns.addRecords(records={"foo.com.": ["127.0.0.1"]})
dns.addRecords(records={"bar.com.": ["127.0.0.1"]})

foo_ec_string = ""
foo_rsa_string = ""
san_ec_string = ""
san_rsa_string = ""
combo_ec_string = ""
combo_rsa_string = ""
with open(os.path.join(Test.TestDirectory, 'ssl', 'signed-foo-ec.pem'), 'r') as myfile:
    file_string = myfile.read()
    cert_end = file_string.find("END CERTIFICATE-----")
    foo_ec_string = re.escape(file_string[0:cert_end])
with open(os.path.join(Test.TestDirectory, 'ssl', 'signed-foo.pem'), 'r') as myfile:
    file_string = myfile.read()
    cert_end = file_string.find("END CERTIFICATE-----")
    foo_rsa_string = re.escape(file_string[0:cert_end])
with open(os.path.join(Test.TestDirectory, 'ssl', 'signed-san-ec.pem'), 'r') as myfile:
    file_string = myfile.read()
    cert_end = file_string.find("END CERTIFICATE-----")
    san_ec_string = re.escape(file_string[0:cert_end])
with open(os.path.join(Test.TestDirectory, 'ssl', 'signed-san.pem'), 'r') as myfile:
    file_string = myfile.read()
    cert_end = file_string.find("END CERTIFICATE-----")
    san_rsa_string = re.escape(file_string[0:cert_end])
with open(os.path.join(Test.TestDirectory, 'ssl', 'combined-ec.pem'), 'r') as myfile:
    file_string = myfile.read()
    cert_end = file_string.find("END CERTIFICATE-----")
    combo_ec_string = re.escape(file_string[0:cert_end])
with open(os.path.join(Test.TestDirectory, 'ssl', 'combined.pem'), 'r') as myfile:
    file_string = myfile.read()
    cert_end = file_string.find("END CERTIFICATE-----")
    combo_rsa_string = re.escape(file_string[0:cert_end])

# Should receive a EC cert since ATS cipher list prefers EC
tr = Test.AddTestRun("Default for foo should return EC cert")
tr.Setup.Copy("ssl/signer.pem")
tr.Processes.Default.Command = "echo foo | openssl s_client -tls1_2 -servername foo.com -connect 127.0.0.1:{0}".format(
    ts.Variables.ssl_port, foo_ec_string)
tr.ReturnCode = 0
tr.Processes.Default.StartBefore(server)
tr.Processes.Default.StartBefore(dns)
tr.Processes.Default.StartBefore(Test.Processes.ts, ready=When.PortOpen(ts.Variables.ssl_port))
tr.StillRunningAfter = server
tr.StillRunningAfter = ts
tr.Processes.Default.Streams.All += Testers.ContainsExpression(foo_ec_string, "Should select EC cert", reflags=re.S | re.M)

# Should receive a RSA cert
tr = Test.AddTestRun("Only offer RSA ciphers, should receive RSA cert")
tr.Processes.Default.Command = "echo foo | openssl s_client -tls1_2 -servername foo.com -cipher 'ECDHE-RSA-AES128-GCM-SHA256' -connect 127.0.0.1:{0}".format(
    ts.Variables.ssl_port)
tr.ReturnCode = 0
tr.StillRunningAfter = server
tr.StillRunningAfter = ts
tr.Processes.Default.Streams.All += Testers.ContainsExpression(foo_rsa_string, "Should select RSA cert", reflags=re.S | re.M)

# Should receive a EC cert
tr = Test.AddTestRun("Default for two.com should return EC cert")
tr.Processes.Default.Command = "echo foo | openssl s_client -tls1_2 -servername two.com -connect 127.0.0.1:{0}".format(
    ts.Variables.ssl_port)
tr.ReturnCode = 0
tr.StillRunningAfter = server
tr.StillRunningAfter = ts
tr.Processes.Default.Streams.All += Testers.ContainsExpression(san_ec_string, "Should select EC cert", reflags=re.S | re.M)
tr.Processes.Default.Streams.All += Testers.ContainsExpression("CN ?= ?group.com", "Should select a group SAN")

# Should receive a RSA cert
tr = Test.AddTestRun("Only offer RSA ciphers, should receive RSA cert")
tr.Processes.Default.Command = "echo foo | openssl s_client -tls1_2 -servername two.com -cipher 'ECDHE-RSA-AES128-GCM-SHA256' -connect 127.0.0.1:{0}".format(
    ts.Variables.ssl_port)
tr.ReturnCode = 0
tr.StillRunningAfter = server
tr.StillRunningAfter = ts
tr.Processes.Default.Streams.All += Testers.ContainsExpression(san_rsa_string, "Should select RSA cert", reflags=re.S | re.M)
tr.Processes.Default.Streams.All += Testers.ContainsExpression("CN ?= ?group.com", "Should select a group SAN")

# Should receive a RSA cert
tr = Test.AddTestRun("rsa.com only in rsa cert")
tr.Processes.Default.Command = "echo foo | openssl s_client -tls1_2 -servername rsa.com -connect 127.0.0.1:{0}".format(
    ts.Variables.ssl_port)
tr.ReturnCode = 0
tr.StillRunningAfter = server
tr.StillRunningAfter = ts
tr.Processes.Default.Streams.All += Testers.ContainsExpression(san_rsa_string, "Should select RSA cert", reflags=re.S | re.M)
tr.Processes.Default.Streams.All += Testers.ContainsExpression("CN ?= ?group.com", "Should select a group SAN")

# Should receive a EC cert
tr = Test.AddTestRun("ec.com only in ec cert")
tr.Processes.Default.Command = "echo foo | openssl s_client -tls1_2 -servername ec.com -connect 127.0.0.1:{0}".format(
    ts.Variables.ssl_port)
tr.ReturnCode = 0
tr.StillRunningAfter = server
tr.StillRunningAfter = ts
tr.Processes.Default.Streams.All += Testers.ContainsExpression(san_ec_string, "Should select EC cert", reflags=re.S | re.M)
tr.Processes.Default.Streams.All += Testers.ContainsExpression("CN ?= ?group.com", "Should select a group SAN")

# Should receive a EC cert
tr = Test.AddTestRun("Default for combined.com should return EC cert")
tr.Processes.Default.Command = "echo foo | openssl s_client -tls1_2 -servername combined.com -connect 127.0.0.1:{0}".format(
    ts.Variables.ssl_port)
tr.ReturnCode = 0
tr.StillRunningAfter = server
tr.StillRunningAfter = ts
tr.Processes.Default.Streams.All += Testers.ContainsExpression(combo_ec_string, "Should select EC cert", reflags=re.S | re.M)
tr.Processes.Default.Streams.All += Testers.ContainsExpression("CN ?= ?combined.com", "Should select combined pem")

# Should receive a RSA cert
tr = Test.AddTestRun("Only offer RSA ciphers, should receive RSA cert")
tr.Processes.Default.Command = "echo foo | openssl s_client -tls1_2 -servername combined.com -cipher 'ECDHE-RSA-AES128-GCM-SHA256' -connect 127.0.0.1:{0}".format(
    ts.Variables.ssl_port)
tr.ReturnCode = 0
tr.StillRunningAfter = server
tr.StillRunningAfter = ts
tr.Processes.Default.Streams.All += Testers.ContainsExpression(combo_rsa_string, "Should select RSA cert", reflags=re.S | re.M)
tr.Processes.Default.Streams.All += Testers.ContainsExpression("CN ?= ?combined.com", "Should select combined pem")



'''
'''
#  Licensed to the Apache Software Foundation (ASF) under one
#  or more contributor license agreements.  See the NOTICE file
#  distributed with this work for additional information
#  regarding copyright ownership.  The ASF licenses this file
#  to you under the Apache License, Version 2.0 (the
#  "License"); you may not use this file except in compliance
#  with the License.  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

import os
import ports

Test.Summary = '''
Forwarding a non-HTTP protocol out of TLS
'''

# Define default ATS
ts = Test.MakeATSProcess("ts", enable_tls=True)

# add ssl materials like key, certificates for the server
ts.addSSLfile("ssl/server.pem")
ts.addSSLfile("ssl/server.key")

# reserve a port of the s_client that will be released with 'ts'
ports.get_port(ts, 's_client_port')

nameserver = Test.MakeDNServer("dns", default='127.0.0.1')

# Need no remap rules.  Everything should be processed by sni

# Make sure the TS server certs are different from the origin certs
ts.Disk.ssl_multicert_config.AddLine('dest_ip=* ssl_cert_name=server.pem ssl_key_name=server.key')

# Case 1, global config policy=permissive properties=signature
#         override for foo.com policy=enforced properties=all
ts.Disk.records_config.update(
    {
        'proxy.config.ssl.server.cert.path': '{0}'.format(ts.Variables.SSLDir),
        'proxy.config.ssl.server.private_key.path': '{0}'.format(ts.Variables.SSLDir),
        'proxy.config.http.connect_ports': '{0} {1}'.format(ts.Variables.ssl_port, ts.Variables.s_client_port),
        'proxy.config.exec_thread.autoconfig.scale': 1.0,
        'proxy.config.url_remap.pristine_host_hdr': 1,
        'proxy.config.dns.nameservers': f"127.0.0.1:{nameserver.Variables.Port}",
        'proxy.config.dns.resolv_conf': 'NULL'
    })

# foo.com should not terminate.  Just tunnel to server_foo
# bar.com should terminate.  Forward its tcp stream to server_bar
ts.Disk.sni_yaml.AddLines([
    "sni:",
    "- fqdn: bar.com",
    "  forward_route: localhost:{0}".format(ts.Variables.s_client_port),
])

tr = Test.AddTestRun("forward-non-http")
tr.Setup.Copy("test-nc-s_client.sh")
cmd_args = ["sh", "test-nc-s_client.sh", str(ts.Variables.s_client_port), str(ts.Variables.ssl_port)]
if Condition.HasOpenSSLVersion("3.0.0"):
    cmd_args += ["-ignore_unexpected_eof"]
tr.Processes.Default.Command = " ".join(cmd_args)
tr.ReturnCode = 0
tr.Processes.Default.StartBefore(nameserver)
tr.Processes.Default.StartBefore(Test.Processes.ts)
tr.StillRunningAfter = ts
testout_path = os.path.join(Test.RunDirectory, "test.out")
tr.Disk.File(testout_path, id="testout")
tr.Processes.Default.Streams.All += Testers.IncludesExpression("This is a reply", "s_client should get response")



"""A gRPC server."""

#  Licensed to the Apache Software Foundation (ASF) under one
#  or more contributor license agreements.  See the NOTICE file
#  distributed with this work for additional information
#  regarding copyright ownership.  The ASF licenses this file
#  to you under the Apache License, Version 2.0 (the
#  "License"); you may not use this file except in compliance
#  with the License.  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

import argparse
import asyncio
from concurrent import futures
import grpc
import sys
import time

import simple_pb2
import simple_pb2_grpc

global_message_counter: int = 0


class Talker(simple_pb2_grpc.TalkerServicer):
    """A gRPC servicer."""

    async def MakeRequest(self, request: simple_pb2.SimpleRequest, context: grpc.aio.ServicerContext):
        """An example gRPC method."""
        global global_message_counter
        global_message_counter += 1
        print(f'Received request: {request.message}')
        response = simple_pb2.SimpleResponse(message=f"Echo: {request.message}")
        return response

    async def MakeAnotherRequest(self, request: simple_pb2.SimpleRequest, context: grpc.aio.ServicerContext):
        """An example gRPC method."""
        global global_message_counter
        global_message_counter += 1
        print(f'Received another request: {request.message}')
        response = simple_pb2.SimpleResponse(message=f"Another echo: {request.message}")
        return response


async def run_grpc_server(port: int, server_cert: str, server_key: str) -> int:
    """Run the gRPC server.

    :param port: The port on which to listen.
    :param server_cert: The public TLS certificate to use.
    :param server_key: The private TLS key to use.
    :return: The exit code.
    """
    credentials = grpc.ssl_server_credentials([(server_key, server_cert)])
    server = grpc.aio.server(futures.ThreadPoolExecutor(max_workers=10))
    simple_pb2_grpc.add_TalkerServicer_to_server(Talker(), server)
    server_endpoint = f'127.0.0.1:{port}'
    server.add_secure_port(server_endpoint, credentials)
    print(f'Listening on: {server_endpoint}')
    try:
        await server.start()
        await server.wait_for_termination()
    except asyncio.exceptions.CancelledError:
        print('Shutting down the server.')
    finally:
        await server.stop(0)
    return 0


def parse_args() -> argparse.Namespace:
    """Parse command line arguments."""
    parser = argparse.ArgumentParser(description=__doc__)
    parser.add_argument('port', type=int, help='The port on which to listen.')
    parser.add_argument('server_crt', metavar='server-crt', type=argparse.FileType('rb'), help="The public TLS certificate to use.")
    parser.add_argument('server_key', metavar='server-key', type=argparse.FileType('rb'), help="The private TLS key to use.")
    parser.add_argument(
        'num_expected_messages', metavar='num-expected-messages', type=int, help="The number of expected messages from the client.")
    return parser.parse_args()


def main() -> int:
    """Run the main entry point for the gRPC server.

    :return: The exit code.
    """
    args = parse_args()
    try:
        return asyncio.run(run_grpc_server(args.port, args.server_crt.read(), args.server_key.read()))
    except KeyboardInterrupt:
        pass

    if global_message_counter == args.num_expected_messages:
        print(f'Received {args.num_expected_messages} messages as expected.')
        return 0
    else:
        print(f'Expected {args.num_expected_messages} messages, but received {global_message_counter}.')
        return 1


if __name__ == '__main__':
    sys.exit(main())



'''
'''
#  Licensed to the Apache Software Foundation (ASF) under one
#  or more contributor license agreements.  See the NOTICE file
#  distributed with this work for additional information
#  regarding copyright ownership.  The ASF licenses this file
#  to you under the Apache License, Version 2.0 (the
#  "License"); you may not use this file except in compliance
#  with the License.  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

Test.Summary = 'Testing ATS TCP handshake timeout'

# Skipping this in the normal CI because it requires privilege.
# Comment out to run in your privileged environment
Test.SkipIf(Condition.true("Test requires privilege"))

ts = Test.MakeATSProcess("ts")

Test.ContinueOnFail = True
Test.GetTcpPort("blocked_upstream_port")
Test.GetTcpPort("upstream_port")

server4 = Test.MakeOriginServer("server4")

ts.Disk.records_config.update(
    {
        'proxy.config.url_remap.remap_required': 1,
        'proxy.config.http.connect_attempts_timeout': 2,
        'proxy.config.http.connect_attempts_max_retries': 0,
        'proxy.config.http.transaction_no_activity_timeout_out': 5,
        'proxy.config.diags.debug.enabled': 0,
        'proxy.config.diags.debug.tags': 'http',
    })

ts.Disk.remap_config.AddLine('map /blocked http://10.1.1.1:{0}'.format(Test.Variables.blocked_upstream_port))
ts.Disk.remap_config.AddLine('map /not-blocked http://10.1.1.1:{0}'.format(Test.Variables.upstream_port))

ts.Disk.logging_yaml.AddLines(
    '''
logging:
  formats:
    - name: testformat
      format: '%<pssc> %<pquc> %<pscert> %<cscert>'
  logs:
    - mode: ascii
      format: testformat
      filename: squid
'''.split("\n"))

# Set up the network name space.  Requires privilege
tr = Test.AddTestRun("tr-ns-setup")
tr.Processes.Default.StartBefore(ts)
tr.Processes.Default.TimeOut = 2
tr.Setup.Copy('setupnetns.sh')
tr.Processes.Default.Command = 'echo start; sudo sh -x ./setupnetns.sh {0} {1}'.format(
    Test.Variables.blocked_upstream_port, Test.Variables.upstream_port)

# Request to the port that is blocked in the network ns.  The SYN should never be responded to
# and the connect timeout should trigger with a 50x return.  If the SYN handshake occurs, the
# no activity timeout would trigger, but not before the test timeout expires
tr = Test.AddTestRun("tr-blocking")
tr.Processes.Default.Command = 'curl -i http://127.0.0.1:{0}/blocked {0}'.format(ts.Variables.port)
tr.Processes.Default.TimeOut = 4
tr.Processes.Default.Streams.All = Testers.ContainsExpression(
    "HTTP/1.1 502 internal error - server connection terminated", "Connect failed")

tr = Test.AddTestRun("tr-blocking-post")
tr.Processes.Default.Command = 'curl -d "stuff" -i http://127.0.0.1:{0}/blocked {0}'.format(ts.Variables.port)
tr.Processes.Default.TimeOut = 4
tr.Processes.Default.Streams.All = Testers.ContainsExpression(
    "HTTP/1.1 502 internal error - server connection terminated", "Connect failed")

#  Should not catch the connect timeout.  Even though the first bytes are not sent until after the 2 second connect timeout
#  But before the no-activity timeout
tr = Test.AddTestRun("tr-delayed")
tr.Setup.Copy('delay-server.sh')
tr.Setup.Copy('case1.sh')
tr.Processes.Default.Command = 'sh ./case1.sh {0} {1}'.format(ts.Variables.port, ts.Variables.upstream_port)
tr.Processes.Default.TimeOut = 7
tr.Processes.Default.Streams.All = Testers.ContainsExpression("HTTP/1.1 200", "Connect succeeded")

# cleanup the network namespace and virtual network
tr = Test.AddTestRun("tr-cleanup")
tr.Processes.Default.Command = 'sudo ip netns del testserver; sudo ip link del veth0 type veth peer name veth1'
tr.Processes.Default.TimeOut = 4

tr = Test.AddTestRun("Wait for the access log to write out")
tr.Processes.Default.StartBefore(server4, ready=When.FileExists(ts.Disk.squid_log))
tr.StillRunningAfter = ts
tr.Processes.Default.Command = 'echo "log file exists"'
tr.Processes.Default.ReturnCode = 0



# @file
#
#  Licensed to the Apache Software Foundation (ASF) under one
#  or more contributor license agreements.  See the NOTICE file
#  distributed with this work for additional information
#  regarding copyright ownership.  The ASF licenses this file
#  to you under the Apache License, Version 2.0 (the
#  "License"); you may not use this file except in compliance
#  with the License.  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.
#
#  Copyright 2021, Yahoo Inc
#
Test.Summary = '''
txn-debug directive
'''

Test.SkipUnless(Condition.PluginExists("txn_box.so"))
Test.SkipIf(Condition.true("This needs to be revisit. TS not finishing up gracefully."))

replay_file = "txn-debug.replay.yaml"

tr = Test.TxnBoxTestAndRun(
    "Test txn-debug enabled",
    replay_file,
    config_path='Auto',
    verifier_client_args="--verbose diag --keys debug-expected",
    verifier_server_args="--verbose diag",
    config_key="meta.txn_box.global",
    suffix="debug-enabled")
ts = tr.Variables.TS
ts.Disk.records_config.update(
    {
        'proxy.config.diags.debug.enabled': 1,
        'proxy.config.diags.debug.tags': 'txn_box',
        'proxy.config.crash_log_helper':
            '/home/dmeden/code/git/trafficserver/build/_sandbox/txn-debug/ts-debug-enabled/bin/traffic_crashlog'
    })

ts.Disk.traffic_out.Content += Testers.ContainsExpression(r"DIAG: <HttpSM.cc", "Verify that there was transaction level debugging.")

tr = Test.TxnBoxTestAndRun(
    "Test txn-debug disabled",
    replay_file,
    config_path='Auto',
    verifier_client_args="--verbose diag --keys debug-not-expected",
    config_key="meta.txn_box.global",
    suffix="debug-disabled")
ts = tr.Variables.TS
ts.Disk.records_config.update(
    {
        'proxy.config.diags.debug.enabled': 1,
        'proxy.config.diags.debug.tags': 'txn_box',
        'proxy.config.crash_log_helper':
            '/home/dmeden/code/git/trafficserver/build/_sandbox/txn-debug/ts-debug-disabled/bin/traffic_crashlog'
    })

ts.Disk.traffic_out.Content += Testers.ExcludesExpression(
    r"DIAG: <HttpSM.cc", "Verify that there was not transaction level debugging.")



'''
'''
#  Licensed to the Apache Software Foundation (ASF) under one
#  or more contributor license agreements.  See the NOTICE file
#  distributed with this work for additional information
#  regarding copyright ownership.  The ASF licenses this file
#  to you under the Apache License, Version 2.0 (the
#  "License"); you may not use this file except in compliance
#  with the License.  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.
import re

Test.Summary = '''
AuTest with bad configuration of microserver to simulate server aborting the connection unexpectedly
'''
ts = Test.MakeATSProcess("ts", enable_tls=True)
# note the microserver by default is not configured to use ssl
server = Test.MakeOriginServer("server")
ts.Disk.remap_config.AddLine(
    # The following config tells ATS to do tls with the origin server on a
    # non-tls port. This is misconfigured intentionally to trigger an exception
    # on the origin server so that it aborts the connection upon receiving a
    # request
    'map / https://127.0.0.1:{0}'.format(server.Variables.Port))
ts.Disk.ssl_multicert_config.AddLine('dest_ip=* ssl_cert_name=aaa-signed.pem ssl_key_name=aaa-signed.key')
ts.Disk.records_config.update(
    {
        'proxy.config.diags.debug.tags': 'http|dns',
        'proxy.config.diags.debug.enabled': 1,
        'proxy.config.ssl.server.cert.path': f'{Test.TestDirectory}/test_secrets',
        'proxy.config.ssl.server.private_key.path': f'{Test.TestDirectory}/test_secrets',
    })

tr = Test.AddTestRun()
tr.Processes.Default.StartBefore(server)
tr.Processes.Default.StartBefore(ts)
tr.Processes.Default.Command = "curl -v -k -H \"host: foo.com\" https://127.0.0.1:{0}".format(ts.Variables.ssl_port)
tr.ReturnCode = 0
tr.StillRunningAfter = server
tr.StillRunningAfter = ts
server.Streams.stderr += Testers.ContainsExpression(
    "UnicodeDecodeError", "Verify that the server raises an exception when processing the request.")



# @file
#
#  Licensed to the Apache Software Foundation (ASF) under one
#  or more contributor license agreements.  See the NOTICE file
#  distributed with this work for additional information
#  regarding copyright ownership.  The ASF licenses this file
#  to you under the Apache License, Version 2.0 (the
#  "License"); you may not use this file except in compliance
#  with the License.  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.
#
#  Copyright 2020, Verizon Media
#
'''
Txn Open (start) hook testing.
'''
Test.Summary = '''
Test transaction start hook.
'''

Test.SkipUnless(Condition.PluginExists("txn_box.so"))

tr = Test.TxnBoxTestAndRun("Test txn_open 1", "txn_open_1.replay.yaml", config_path='Auto', config_key="meta.txn_box.global")

ts = tr.Variables.TS
ts.Disk.records_config.update(
    {
        'proxy.config.log.max_secs_per_buffer': 1,
        'proxy.config.http.transaction_no_activity_timeout_out': 120,
        'proxy.config.http.per_server.connection.max': 500,
        'proxy.config.http.background_fill_completed_threshold': 0.4,
        'proxy.config.diags.debug.enabled': 1,
        'proxy.config.diags.debug.tags': 'txn_box'
    })



'''
Test header_rewrite with URL conditions and operators.
'''
#  Licensed to the Apache Software Foundation (ASF) under one
#  or more contributor license agreements.  See the NOTICE file
#  distributed with this work for additional information
#  regarding copyright ownership.  The ASF licenses this file
#  to you under the Apache License, Version 2.0 (the
#  "License"); you may not use this file except in compliance
#  with the License.  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

Test.Summary = '''
Test header_rewrite with URL conditions and operators.
'''

Test.ContinueOnFail = True
ts = Test.MakeATSProcess("ts")
server = Test.MakeOriginServer("server")

# Configure the server to return 200 responses. The rewrite rules below set a
# non-200 status, so if curl gets a 200 response something went wrong.
Test.testName = ""
request_header = {"headers": "GET / HTTP/1.1\r\nHost: www.example.com\r\n\r\n", "timestamp": "1469733493.993", "body": ""}
response_header = {"headers": "HTTP/1.1 200 OK\r\nConnection: close\r\n\r\n", "timestamp": "1469733493.993", "body": ""}
server.addResponse("sessionfile.log", request_header, response_header)
request_header = {"headers": "GET / HTTP/1.1\r\nHost: no_path.com\r\n\r\n", "timestamp": "1469733493.993", "body": ""}
server.addResponse("sessionfile.log", request_header, response_header)

ts.Disk.records_config.update(
    {
        'proxy.config.diags.debug.enabled': 1,
        'proxy.config.diags.show_location': 0,
        'proxy.config.diags.debug.tags': 'header.*',
    })
# The following rule changes the status code returned from origin server to 303
ts.Setup.CopyAs('rules/rule_client.conf', Test.RunDirectory)
ts.Setup.CopyAs('rules/set_redirect.conf', Test.RunDirectory)

# This configuration makes use of CLIENT-URL in conditions.
ts.Disk.remap_config.AddLine(
    'map http://www.example.com/from_path/ https://127.0.0.1:{0}/to_path/ '
    '@plugin=header_rewrite.so @pparam={1}/rule_client.conf'.format(server.Variables.Port, Test.RunDirectory))
ts.Disk.remap_config.AddLine(
    'map http://www.example.com:8080/from_path/ https://127.0.0.1:{0}/to_path/ '
    '@plugin=header_rewrite.so @pparam={1}/rule_client.conf'.format(server.Variables.Port, Test.RunDirectory))
# This configuration makes use of TO-URL in a set-redirect operator.
ts.Disk.remap_config.AddLine(
    'map http://no_path.com http://no_path.com?name=brian/ '
    '@plugin=header_rewrite.so @pparam={0}/set_redirect.conf'.format(Test.RunDirectory))

# Test CLIENT-URL.
tr = Test.AddTestRun()
tr.Processes.Default.Command = (
    'curl --proxy 127.0.0.1:{0} "http://www.example.com/from_path/hello?=foo=bar" '
    '-H "Proxy-Connection: keep-alive" --verbose'.format(ts.Variables.port))
tr.Processes.Default.ReturnCode = 0
tr.Processes.Default.StartBefore(server, ready=When.PortOpen(server.Variables.Port))
tr.Processes.Default.StartBefore(Test.Processes.ts)
tr.Processes.Default.Streams.stderr = "gold/header_rewrite-client.gold"
tr.StillRunningAfter = server
ts.Disk.traffic_out.Content = "gold/header_rewrite-tag.gold"

# Test TO-URL in a set-redirect operator.
tr = Test.AddTestRun()
tr.Processes.Default.Command = 'curl --head 127.0.0.1:{0} -H "Host: no_path.com" --verbose'.format(ts.Variables.port)
tr.Processes.Default.ReturnCode = 0
tr.Processes.Default.Streams.stderr = "gold/set-redirect.gold"
tr.StillRunningAfter = server
ts.Disk.traffic_out.Content = "gold/header_rewrite-tag.gold"



'''
'''
#  Licensed to the Apache Software Foundation (ASF) under one
#  or more contributor license agreements.  See the NOTICE file
#  distributed with this work for additional information
#  regarding copyright ownership.  The ASF licenses this file
#  to you under the Apache License, Version 2.0 (the
#  "License"); you may not use this file except in compliance
#  with the License.  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

import os
import sys

Test.Summary = '''
Test combo_handler plugin
'''

# Skip if plugin not present.
#
Test.SkipUnless(Condition.PluginExists('combo_handler.so'),)

# Function to generate a unique data file path (in the top level of the test's run directory),  put data (in string 'data') into
# the file, and return the file name.
#
_data_file__file_count = 0


def data_file(data):
    global _data_file__file_count
    file_path = Test.RunDirectory + f"/tcp_client_in_{_data_file__file_count}"
    _data_file__file_count += 1
    with open(file_path, "x") as f:
        f.write(data)
    return file_path


# Function to return command (string) to run tcp_client.py tool.  'host' 'port', and 'file_path' are the parameters to tcp_client.
#


def tcp_client_cmd(host, port, file_path):
    return f"{sys.executable} {Test.Variables.AtsTestToolsDir}/tcp_client.py {host} {port} {file_path}"


# Function to return command (string) to run tcp_client.py tool.  'host' and 'port' are the first two parameters to tcp_client.
# 'data' is the data to put in the data file input to tcp_client.
#


def tcp_client(host, port, data):
    return tcp_client_cmd(host, port, data_file(data))


server = Test.MakeOriginServer("server")


def add_server_obj(content_type, path):
    request_header = {
        "headers": "GET " + path + " HTTP/1.1\r\n" + "Host: just.any.thing\r\n\r\n",
        "timestamp": "1469733493.993",
        "body": ""
    }
    response_header = {
        "headers":
            "HTTP/1.1 200 OK\r\n" + "Connection: close\r\n" + 'Etag: "359670651"\r\n' +
            "Cache-Control: public, max-age=31536000\r\n" + "Accept-Ranges: bytes\r\n" + "Content-Type: " + content_type + "\r\n" +
            "\r\n",
        "timestamp": "1469733493.993",
        "body": "Content for " + path + "\n"
    }
    server.addResponse("sessionfile.log", request_header, response_header)


add_server_obj("text/css ; charset=utf-8", "/obj1")
add_server_obj("text/javascript", "/sub/obj2")
add_server_obj("text/argh", "/obj3")
add_server_obj("application/javascript", "/obj4")

ts = Test.MakeATSProcess("ts")

ts.Disk.records_config.update({
    'proxy.config.diags.debug.enabled': 1,
    'proxy.config.diags.debug.tags': 'http|combo_handler',
})

ts.Disk.plugin_config.AddLine("combo_handler.so - - - ctwl.txt")

ts.Disk.remap_config.AddLine('map http://xyz/ http://127.0.0.1/ @plugin=combo_handler.so')
ts.Disk.remap_config.AddLine(f'map http://localhost/127.0.0.1/ http://127.0.0.1:{server.Variables.Port}/')
ts.Disk.remap_config.AddLine(f'map http://localhost/sub/ http://127.0.0.1:{server.Variables.Port}/sub/')

# Configure the combo_handler's configuration file.
ts.Setup.Copy("ctwl.txt", ts.Variables.CONFIGDIR)

tr = Test.AddTestRun()
tr.Processes.Default.StartBefore(ts)
tr.Processes.Default.StartBefore(server)
tr.Processes.Default.Command = "echo start stuff"
tr.Processes.Default.ReturnCode = 0

tr = Test.AddTestRun()
tr.Processes.Default.Command = tcp_client(
    "127.0.0.1", ts.Variables.port,
    "GET /admin/v1/combo?obj1&sub:obj2&obj3 HTTP/1.1\n" + "Host: xyz\n" + "Connection: close\n" + "\n")
tr.Processes.Default.ReturnCode = 0
f = tr.Disk.File("_output/1-tr-Default/stream.all.txt")
f.Content = "combo_handler_files/tr1.gold"

tr = Test.AddTestRun()
tr.Processes.Default.Command = tcp_client(
    "127.0.0.1", ts.Variables.port,
    "GET /admin/v1/combo?obj1&sub:obj2&obj4 HTTP/1.1\n" + "Host: xyz\n" + "Connection: close\n" + "\n")
tr.Processes.Default.ReturnCode = 0
f = tr.Disk.File("_output/2-tr-Default/stream.all.txt")
f.Content = "combo_handler_files/tr2.gold"

ts.Disk.diags_log.Content = Testers.ContainsExpression("ERROR", "Some tests are failure tests")



#!/usr/bin/env python3
"""A server that replies without waiting for the entire request."""

#  Licensed to the Apache Software Foundation (ASF) under one
#  or more contributor license agreements.  See the NOTICE file
#  distributed with this work for additional information
#  regarding copyright ownership.  The ASF licenses this file
#  to you under the Apache License, Version 2.0 (the
#  "License"); you may not use this file except in compliance
#  with the License.  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

from http_utils import (wait_for_headers_complete, determine_outstanding_bytes_to_read, drain_socket)

import argparse
import socket
import sys


def parse_args() -> argparse.Namespace:
    """Parse command line arguments."""
    parser = argparse.ArgumentParser(description=__doc__)
    parser.add_argument("address", help="Address to listen on")
    parser.add_argument("port", type=int, default=8080, help="The port to listen on")
    parser.add_argument('--drain-request', action='store_true', help="Drain the entire request before closing the connection")
    parser.add_argument(
        '--abort-response-headers', action='store_true', help="Abort the response in the midst of sending the response headers")
    return parser.parse_args()


def get_listening_socket(address: str, port: int) -> socket.socket:
    """Create a listening socket.

    :param address: The address to listen on.
    :param port: The port to listen on.
    :returns: A listening socket.
    """
    sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    sock.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
    sock.bind((address, port))
    sock.listen(1)
    return sock


def accept_connection(sock: socket.socket) -> socket.socket:
    """Accept a connection.

    :param sock: The socket to accept a connection on.
    :returns: The accepted socket.
    """
    return sock.accept()[0]


def send_response(sock: socket.socket, abort_early: bool) -> None:
    """Send an HTTP response.

    :param sock: The socket to write to.
    :param abort_early: If True, abort the response before sending the body.
    """
    if abort_early:
        response = "HTTP/1."
    else:
        response = ("HTTP/1.1 200 OK\r\n"
                    "Content-Length: 0\r\n"
                    "\r\n")
    print(f'Sending:\n{response}')
    sock.sendall(response.encode("utf-8"))


def main() -> int:
    """Run the server."""
    args = parse_args()

    # Configure a listening socket on args.address and args.port.
    with get_listening_socket(args.address, args.port) as listening_sock:
        print(f"Listening on {args.address}:{args.port}")

        read_bytes: bytes = b""
        while len(read_bytes) == 0:
            with accept_connection(listening_sock) as sock:
                read_bytes = wait_for_headers_complete(sock)
                if len(read_bytes) == 0:
                    # This is probably the PortOpenv4 test. Try again.
                    print("No bytes read on this connection. Trying again.")
                    sock.close()
                    continue

                # Send a response now, before the body is read. This implements
                # the "quick" attribute of this quick_server.
                send_response(sock, args.abort_response_headers)

                if args.abort_response_headers:
                    # We're done.
                    break

                if args.drain_request:
                    num_bytes_to_drain = determine_outstanding_bytes_to_read(read_bytes)
                    print(f'Read {len(read_bytes)} bytes. '
                          f'Draining {num_bytes_to_drain} bytes.')
                    drain_socket(sock, read_bytes, num_bytes_to_drain)
                else:
                    print(f'Read {len(read_bytes)} bytes. '
                          f'Not draining per configuration.')
    return 0


if __name__ == "__main__":
    sys.exit(main())



'''
'''
#  Licensed to the Apache Software Foundation (ASF) under one
#  or more contributor license agreements.  See the NOTICE file
#  distributed with this work for additional information
#  regarding copyright ownership.  The ASF licenses this file
#  to you under the Apache License, Version 2.0 (the
#  "License"); you may not use this file except in compliance
#  with the License.  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

import os
from jsonrpc import Notification, Request, Response

Test.Summary = 'Basic records test. Testing the new records.yaml logic and making sure it works as expected.'

ts = Test.MakeATSProcess("ts")

ts.Disk.records_config.update(
    '''
    accept_threads: 1
    cache:
      limits:
        http:
          max_alts: 5
    diags:
      debug:
        enabled: 0
        tags: http|dns
    ''')

# 0 - We want to make sure that the unregistered records are still being detected.
tr = Test.AddTestRun("Test Append value to existing records.yaml")

tr.Processes.Default.Command = 'traffic_ctl config set proxy.config.diags.debug.tags rpc --cold'
tr.Processes.Default.ReturnCode = 0
tr.Processes.Default.Env = ts.Env
tr.Processes.Default.StartBefore(ts)
ts.Disk.records_config.Content = 'gold/records.yaml.cold_test0.gold'

# 1
tr = Test.AddTestRun("Get value from latest added node.")
tr.Processes.Default.Command = 'traffic_ctl config get proxy.config.diags.debug.tags --cold'
tr.Processes.Default.ReturnCode = 0
tr.Processes.Default.Env = ts.Env

tr.Processes.Default.Streams.stdout += Testers.ContainsExpression(
    'proxy.config.diags.debug.tags: rpc', 'Config should show the right tags')

# 2
tr = Test.AddTestRun("Test modify latest yaml document from records.yaml")
tr.Processes.Default.Command = 'traffic_ctl config set proxy.config.diags.debug.tags http -u -c'
tr.Processes.Default.ReturnCode = 0
tr.Processes.Default.Env = ts.Env
ts.Disk.records_config.Content = 'gold/records.yaml.cold_test2.gold'

# 3
tr = Test.AddTestRun("Get value from latest added node 1.")
tr.Processes.Default.Command = 'traffic_ctl config get proxy.config.diags.debug.tags --cold'
tr.Processes.Default.ReturnCode = 0
tr.Processes.Default.Env = ts.Env

tr.Processes.Default.Streams.stdout += Testers.ContainsExpression(
    'proxy.config.diags.debug.tags: http', 'Config should show the right tags')

# 4
tr = Test.AddTestRun("Append a new field node using a tag")
tr.Processes.Default.Command = 'traffic_ctl config set proxy.config.cache.limits.http.max_alts 1 -t int -c'
tr.Processes.Default.ReturnCode = 0
tr.Processes.Default.Env = ts.Env
ts.Disk.records_config.Content = 'gold/records.yaml.cold_test4.gold'

# 5
file = os.path.join(ts.Variables.CONFIGDIR, "new_records.yaml")
tr = Test.AddTestRun("Adding a new node(with update flag set) to a non existing file")
tr.Processes.Default.Command = f'traffic_ctl config set proxy.config.cache.limits.http.max_alts 3  -u -c {file}'
tr.Processes.Default.ReturnCode = 0
tr.Processes.Default.Env = ts.Env
tr.Disk.File(file).Content = 'gold/records.yaml.cold_test5.gold'

# 5
file = os.path.join(ts.Variables.CONFIGDIR, "new_records2.yaml")
tr = Test.AddTestRun("Adding a new node to a non existing file")
tr.Processes.Default.Command = f'traffic_ctl config set proxy.config.cache.limits.http.max_alts 3 -c {file}'
tr.Processes.Default.ReturnCode = 0
tr.Processes.Default.Env = ts.Env
tr.Disk.File(file).Content = 'gold/records.yaml.cold_test5.gold'



#  Licensed to the Apache Software Foundation (ASF) under one
#  or more contributor license agreements.  See the NOTICE file
#  distributed with this work for additional information
#  regarding copyright ownership.  The ASF licenses this file
#  to you under the Apache License, Version 2.0 (the
#  "License"); you may not use this file except in compliance
#  with the License.  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

Test.Summary = '''
Test remap_stats plugin
'''
# Skip if plugins not present.
Test.SkipUnless(Condition.PluginExists('remap_stats.so'))

server = Test.MakeOriginServer("server")

Test.Setup.Copy("metrics_post.sh")

request_header = {"headers": "GET /argh HTTP/1.1\r\nHost: one\r\n\r\n", "timestamp": "1469733493.993", "body": ""}
response_header = {"headers": "HTTP/1.1 200 OK\r\nConnection: close\r\n\r\n", "timestamp": "1469733493.993", "body": ""}
server.addResponse("sessionlog.json", request_header, response_header)

ts = Test.MakeATSProcess("ts")

ts.Disk.plugin_config.AddLine('remap_stats.so --post-remap-host')

ts.Disk.remap_config.AddLine("map http://one http://127.0.0.1:{0}".format(server.Variables.Port))
ts.Disk.remap_config.AddLine("map http://two http://127.0.0.1:{0}".format(server.Variables.Port))

ts.Disk.records_config.update(
    {
        'proxy.config.http.transaction_active_timeout_out': 2,
        'proxy.config.http.transaction_no_activity_timeout_out': 2,
        'proxy.config.http.connect_attempts_timeout': 2,
    })

# 0 Test - Curl host One
tr = Test.AddTestRun("curl host one")
tr.Processes.Default.StartBefore(server)
tr.Processes.Default.StartBefore(Test.Processes.ts)
tr.Processes.Default.Command = 'curl -o /dev/null -H "Host: one"' + ' http://127.0.0.1:{}/argh'.format(ts.Variables.port)
tr.Processes.Default.ReturnCode = 0
tr.StillRunningAfter = ts
tr.StillRunningAfter = server

# 1 Test - Curl host Two
tr = Test.AddTestRun("curl host two")
tr.Processes.Default.Command = 'curl -o /dev/null -H "Host: two"' + ' http://127.0.0.1:{}/badpath'.format(ts.Variables.port)
tr.Processes.Default.ReturnCode = 0
tr.StillRunningAfter = ts
tr.StillRunningAfter = server

# 2 Test - Gather output
tr = Test.AddTestRun("analyze stats")
tr.Processes.Default.Command = 'bash -c ./metrics_post.sh'
tr.Processes.Default.Env = ts.Env
tr.Processes.Default.ReturnCode = 0
tr.StillRunningAfter = ts



'''
'''
#  Licensed to the Apache Software Foundation (ASF) under one
#  or more contributor license agreements.  See the NOTICE file
#  distributed with this work for additional information
#  regarding copyright ownership.  The ASF licenses this file
#  to you under the Apache License, Version 2.0 (the
#  "License"); you may not use this file except in compliance
#  with the License.  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

Test.Summary = '''
Test tls server certificate verification options. Exercise conf_remap
'''

# Define default ATS
ts = Test.MakeATSProcess("ts")
cafile = "{0}/signer.pem".format(Test.RunDirectory)

server_foo = Test.MakeOriginServer(
    "server_foo",
    ssl=True,
    options={
        "--key": "{0}/signed-foo.key".format(Test.RunDirectory),
        "--cert": "{0}/signed-foo.pem".format(Test.RunDirectory),
        "--clientCA": cafile,
        "--clientverify": ""
    },
    clientcert="{0}/signed-bar.pem".format(Test.RunDirectory),
    clientkey="{0}/signed-bar.key".format(Test.RunDirectory))
server_bar = Test.MakeOriginServer(
    "server_bar",
    ssl=True,
    options={
        "--key": "{0}/signed-foo.key".format(Test.RunDirectory),
        "--cert": "{0}/signed-foo.pem".format(Test.RunDirectory),
        "--clientCA": cafile,
        "--clientverify": ""
    },
    clientcert="{0}/signed-bar.pem".format(Test.RunDirectory),
    clientkey="{0}/signed-bar.key".format(Test.RunDirectory))

dns = Test.MakeDNServer("dns")

request_foo_header = {"headers": "GET / HTTP/1.1\r\nHost: foo.com\r\n\r\n", "timestamp": "1469733493.993", "body": ""}
request_bad_foo_header = {"headers": "GET / HTTP/1.1\r\nHost: bad_foo.com\r\n\r\n", "timestamp": "1469733493.993", "body": ""}
request_bar_header = {"headers": "GET / HTTP/1.1\r\nHost: bar.com\r\n\r\n", "timestamp": "1469733493.993", "body": ""}
request_bad_bar_header = {"headers": "GET / HTTP/1.1\r\nHost: bad_bar.com\r\n\r\n", "timestamp": "1469733493.993", "body": ""}
response_header = {"headers": "HTTP/1.1 200 OK\r\nConnection: close\r\n\r\n", "timestamp": "1469733493.993", "body": ""}
server_foo.addResponse("sessionlog.json", request_foo_header, response_header)
server_foo.addResponse("sessionlog.json", request_bad_foo_header, response_header)
server_bar.addResponse("sessionlog.json", request_bar_header, response_header)
server_bar.addResponse("sessionlog.json", request_bad_bar_header, response_header)

server_bar.Setup.Copy("ssl/signer.pem")
server_bar.Setup.Copy("ssl/signer2.pem")
server_foo.Setup.Copy("ssl/signer.pem")
server_foo.Setup.Copy("ssl/signer2.pem")

# add ssl materials like key, certificates for the server
ts.addSSLfile("ssl/signed-foo.pem")
ts.addSSLfile("ssl/signed-foo.key")
ts.addSSLfile("ssl/signed-bar.pem")
ts.addSSLfile("ssl/signed-bar.key")
ts.addSSLfile("ssl/server.pem")
ts.addSSLfile("ssl/server.key")
ts.addSSLfile("ssl/signer.pem")
ts.addSSLfile("ssl/signer.key")

ts.Disk.remap_config.AddLine('map http://foo.com/defaultbar https://bar.com:{0}'.format(server_bar.Variables.SSL_Port))
ts.Disk.remap_config.AddLine('map http://foo.com/default https://foo.com:{0}'.format(server_foo.Variables.SSL_Port))
ts.Disk.remap_config.AddLine(
    'map http://foo.com/overridepolicy https://bar.com:{0} @plugin=conf_remap.so @pparam=proxy.config.ssl.client.verify.server.policy=ENFORCED'
    .format(server_foo.Variables.SSL_Port))
ts.Disk.remap_config.AddLine(
    'map http://foo.com/overrideproperties https://bar.com:{0} @plugin=conf_remap.so @pparam=proxy.config.ssl.client.verify.server.properties=SIGNATURE'
    .format(server_foo.Variables.SSL_Port))

ts.Disk.ssl_multicert_config.AddLine('dest_ip=* ssl_cert_name=server.pem ssl_key_name=server.key')

# global config policy=permissive properties=all
ts.Disk.records_config.update(
    {
        'proxy.config.diags.debug.enabled': 1,
        'proxy.config.diags.debug.tags': 'ssl',
        'proxy.config.ssl.server.cert.path': '{0}'.format(ts.Variables.SSLDir),
        'proxy.config.ssl.server.private_key.path': '{0}'.format(ts.Variables.SSLDir),
        # set global policy
        'proxy.config.ssl.client.verify.server.policy': 'PERMISSIVE',
        'proxy.config.ssl.client.verify.server.properties': 'ALL',
        'proxy.config.ssl.client.CA.cert.path': '{0}'.format(ts.Variables.SSLDir),
        'proxy.config.ssl.client.CA.cert.filename': 'signer.pem',
        'proxy.config.url_remap.pristine_host_hdr': 1,
        'proxy.config.dns.nameservers': '127.0.0.1:{0}'.format(dns.Variables.Port),
        'proxy.config.dns.resolv_conf': 'NULL',
        'proxy.config.exec_thread.autoconfig.scale': 1.0,
        'proxy.config.ssl.client.sni_policy': 'remap'
    })

ts.Disk.sni_yaml.AddLines(
    [
        'sni:',
        '- fqdn: bar.com',
        '  client_cert: "{0}/signed-foo.pem"'.format(ts.Variables.SSLDir),
        '  client_key: "{0}/signed-foo.key"'.format(ts.Variables.SSLDir),
    ])

dns.addRecords(records={"foo.com.": ["127.0.0.1"]})
dns.addRecords(records={"bar.com.": ["127.0.0.1"]})
dns.addRecords(records={"random.com.": ["127.0.0.1"]})

# Should succeed with message
# exercise default settings
tr = Test.AddTestRun("default-permissive-success")
tr.Setup.Copy("ssl/signed-foo.key")
tr.Setup.Copy("ssl/signed-foo.pem")
tr.Setup.Copy("ssl/signed-bar.key")
tr.Setup.Copy("ssl/signed-bar.pem")
tr.Processes.Default.Command = 'curl -k -H \"host: foo.com\"  http://127.0.0.1:{0}/defaultbar'.format(ts.Variables.port)
tr.ReturnCode = 0
tr.Processes.Default.StartBefore(dns)
tr.Processes.Default.StartBefore(server_foo)
tr.Processes.Default.StartBefore(server_bar)
tr.Processes.Default.StartBefore(Test.Processes.ts)
tr.StillRunningAfter = ts
tr.Processes.Default.Streams.stdout = Testers.ExcludesExpression("Could Not Connect", "Curl attempt should have succeeded")

# should fail.  Exercise the override
tr2 = Test.AddTestRun("policy-override-fail")
tr2.Processes.Default.Command = "curl -k -H \"host: foo.com\"  http://127.0.0.1:{0}/overridepolicy".format(ts.Variables.port)
tr2.ReturnCode = 0
tr2.StillRunningAfter = ts
tr2.Processes.Default.Streams.stdout = Testers.ContainsExpression("Could Not Connect", "Curl attempt should fail")

# should succeed with an error message
tr2 = Test.AddTestRun("properties-override-permissive")
tr2.Processes.Default.Command = "curl -k -H \"host: foo.com\"  http://127.0.0.1:{0}/overrideproperties".format(ts.Variables.port)
tr2.ReturnCode = 0
tr2.StillRunningAfter = ts
tr2.Processes.Default.Streams.stdout = Testers.ExcludesExpression("Could Not Connect", "Curl attempt should have succeeded")

# Over riding the built in ERROR check since we expect some cases to fail
ts.Disk.diags_log.Content = Testers.ContainsExpression(
    r"WARNING: SNI \(bar.com\) not in certificate. Action=Continue server=bar.com", "Warning for mismatch name not enforcing")
ts.Disk.diags_log.Content += Testers.ContainsExpression(
    r" WARNING: SNI \(bar.com\) not in certificate. Action=Terminate server=bar.com", "Warning for enforcing mismatch")



'''
Test correct handling of alternates via the Vary header.
'''
#  Licensed to the Apache Software Foundation (ASF) under one
#  or more contributor license agreements.  See the NOTICE file
#  distributed with this work for additional information
#  regarding copyright ownership.  The ASF licenses this file
#  to you under the Apache License, Version 2.0 (the
#  "License"); you may not use this file except in compliance
#  with the License.  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

Test.Summary = '''
Test correct handling of alternates via the Vary header.
'''

ts = Test.MakeATSProcess("ts")
replay_file = "replay/varied_transactions.replay.yaml"
server = Test.MakeVerifierServerProcess("server", replay_file)
ts.Disk.records_config.update(
    {
        'proxy.config.diags.debug.enabled': 1,
        'proxy.config.diags.debug.tags': 'http',
        'proxy.config.http.insert_age_in_response': 0,
        'proxy.config.cache.limits.http.max_alts': 4,
        'proxy.config.cache.log.alternate.eviction': 1,
    })

ts.Disk.remap_config.AddLine('map / http://127.0.0.1:{0}'.format(server.Variables.http_port))

tr = Test.AddTestRun("Run traffic with max_alts behavior when set to 4")
tr.Processes.Default.StartBefore(server)
tr.Processes.Default.StartBefore(ts)
tr.AddVerifierClientProcess("client", replay_file, http_ports=[ts.Variables.port])

ts.Disk.diags_log.Content += "gold/two_alternates_evicted.gold"



'''
'''
#  Licensed to the Apache Software Foundation (ASF) under one
#  or more contributor license agreements.  See the NOTICE file
#  distributed with this work for additional information
#  regarding copyright ownership.  The ASF licenses this file
#  to you under the Apache License, Version 2.0 (the
#  "License"); you may not use this file except in compliance
#  with the License.  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

Test.Summary = '''
Test next hop selection using strategies.yaml with consistent hashing, with peering.
'''

# The tls_conn_timeout test will fail if it runs before this test in CI.  Therefore, this test has a zzz
# prefix so it will run last in CI.

# Define and populate MicroServer.
#
server = Test.MakeOriginServer("server")
response_header = {
    "headers": "HTTP/1.1 200 OK\r\n"
               "Connection: close\r\n"
               "Cache-control: max-age=85000\r\n"
               "\r\n",
    "timestamp": "1469733493.993",
    "body": "This is the body.\n"
}
num_object = 16
for i in range(num_object):
    request_header = {
        "headers":
            f"GET /obj{i} HTTP/1.1\r\n"
            "Host: does.not.matter\r\n"  # But cannot be omitted.
            "\r\n",
        "timestamp": "1469733493.993",
        "body": ""
    }
    server.addResponse("sessionlog.json", request_header, response_header)

dns = Test.MakeDNServer("dns")

# Define upstream trafficserver instances.
#
num_upstream = 6
ts_upstream = []
for i in range(num_upstream):
    ts = Test.MakeATSProcess(f"ts_upstream{i}")
    dns.addRecords(records={f"ts_upstream{i}": ["127.0.0.1"]})
    ts.Disk.records_config.update(
        {
            'proxy.config.diags.debug.enabled': 1,
            'proxy.config.diags.debug.tags': 'http|dns',
            'proxy.config.dns.nameservers': f"127.0.0.1:{dns.Variables.Port}",
            'proxy.config.dns.resolv_conf': "NULL",
        })
    ts.Disk.remap_config.AddLine(f"map / http://127.0.0.1:{server.Variables.Port}")
    ts_upstream.append(ts)

# Define peer trafficserver instances.
#
num_peer = 8
ts_peer = []
for i in range(num_peer):
    ts = Test.MakeATSProcess(f"ts_peer{i}", use_traffic_out=False, command=f"traffic_server 2> trace_peer{i}.log")
    ts.ReturnCode = Any(0, -2)
    ts_peer.append(ts)
for i in range(num_peer):
    ts = ts_peer[i]
    dns.addRecords(records={f"ts_peer{i}": ["127.0.0.1"]})

    ts.Disk.records_config.update(
        {
            'proxy.config.diags.debug.enabled': 1,
            'proxy.config.diags.debug.tags': 'http|dns|parent|next_hop|host_statuses|hostdb|cachekey',
            'proxy.config.dns.nameservers': f"127.0.0.1:{dns.Variables.Port}",  # Only nameservers if resolv_conf NULL.
            'proxy.config.dns.resolv_conf': "NULL",  # This defaults to /etc/resvolv.conf (OS namesevers) if not NULL.
            'proxy.config.http.cache.http': 1,
            'proxy.config.http.cache.required_headers': 0,
            'proxy.config.http.uncacheable_requests_bypass_parent': 0,
            'proxy.config.http.no_dns_just_forward_to_parent': 1,
            'proxy.config.http.parent_proxy.mark_down_hostdb': 0,
            'proxy.config.http.parent_proxy.self_detect': 1,
        })

    ts.Disk.File(ts.Variables.CONFIGDIR + "/strategies.yaml", id="strategies", typename="ats:config")
    s = ts.Disk.strategies
    s.AddLine("groups:")
    s.AddLine("  - &peer_group")
    for j in range(num_peer):
        s.AddLine(f"    - host: ts_peer{j}")
        s.AddLine(f"      protocol:")
        s.AddLine(f"        - scheme: http")
        s.AddLine(f"          port: {ts_peer[j].Variables.port}")
        # The health check URL does not seem to be used currently.
        # s.AddLine(f"          health_check_url: http://ts_peer{j}:{ts_peer[j].Variables.port}")
        s.AddLine(f"      weight: 1.0")
    s.AddLine("  - &peer_upstream")
    for j in range(num_upstream):
        s.AddLine(f"    - host: ts_upstream{j}")
        s.AddLine(f"      protocol:")
        s.AddLine(f"        - scheme: http")
        s.AddLine(f"          port: {ts_upstream[j].Variables.port}")
        # The health check URL does not seem to be used currently.
        # s.AddLine(f"          health_check_url: http://ts_upstream{j}:{ts_upstream[j].Variables.port}")
        s.AddLine(f"      weight: 1.0")
    s.AddLines(
        [
            "strategies:",
            "  - strategy: the-strategy",
            "    policy: consistent_hash",
            "    hash_key: cache_key",
            "    go_direct: false",
            "    parent_is_proxy: true",
            "    cache_peer_result: false",
            "    ignore_self_detect: false",
            "    groups:",
            "      - *peer_group",
            "      - *peer_upstream",
            "    scheme: http",
            "    failover:",
            "      ring_mode: peering_ring",
            f"      self: ts_peer{i}",
            #"      max_simple_retries: 2",
            #"      response_codes:",
            #"        - 404",
            #"      health_check:",
            #"        - passive",
        ])

    suffix = " @strategy=the-strategy @plugin=cachekey.so @pparam=--uri-type=remap @pparam=--capture-prefix=/(.*):(.*)/$1/"
    ts.Disk.remap_config.AddLines(
        [
            "map http://dummy.com http://not_used" + suffix,
            "map http://not_used http://also_not_used" + suffix,
        ])

tr = Test.AddTestRun()
tr.Processes.Default.StartBefore(server)
tr.Processes.Default.StartBefore(dns)
for i in range(num_peer):
    tr.Processes.Default.StartBefore(ts_peer[i])
for i in range(num_upstream):
    tr.Processes.Default.StartBefore(ts_upstream[i])
tr.Processes.Default.Command = 'echo start peer and upstream TSes, HTTP server and DNS server'
tr.Processes.Default.ReturnCode = 0

for i in range(num_object):
    tr = Test.AddTestRun()
    tr.Processes.Default.Command = (
        f'curl --verbose --proxy 127.0.0.1:{ts_peer[i % num_peer].Variables.port} http://dummy.com/obj{i}')
    tr.Processes.Default.Streams.stdout = "body.gold"
    tr.Processes.Default.ReturnCode = 0

for i in range(num_object):
    tr = Test.AddTestRun()
    # num_peer must not be a multiple of 3
    tr.Processes.Default.Command = (
        f'curl --verbose --proxy 127.0.0.1:{ts_peer[(i * 3) % num_peer].Variables.port} http://dummy.com/obj{i}')
    tr.Processes.Default.Streams.stdout = "body.gold"
    tr.Processes.Default.ReturnCode = 0

tr = Test.AddTestRun()
tr.Processes.Default.Command = (
    "grep -e '^+++' -e '^[A-Z].*TTP/' -e '^.alts. --' -e 'PARENT_SPECIFIED' trace_peer*.log"
    " | sed 's/^.*(next_hop) [^ ]* //' | sed 's/[.][0-9]*$$//'")
tr.Processes.Default.Streams.stdout = "trace.gold"
tr.Processes.Default.ReturnCode = 0



'''
Test the allow-plain attribute of the ssl port
Clients sending non-tls request to ssl port should get passed to
non-tls processing in ATS if the allow-plain attibute is present
'''
#  Licensed to the Apache Software Foundation (ASF) under one
#  or more contributor license agreements.  See the NOTICE file
#  distributed with this work for additional information
#  regarding copyright ownership.  The ASF licenses this file
#  to you under the Apache License, Version 2.0 (the
#  "License"); you may not use this file except in compliance
#  with the License.  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

import os

Test.Summary = '''
Test allow-plain attributed
'''

Test.ContinueOnFail = False

# Define default ATS
ts = Test.MakeATSProcess("ts", enable_tls=True)

# ----
# Setup Origin Server
# ----
replay_file = "replay/allow-plain.replay.yaml"
server = Test.MakeVerifierServerProcess("server", replay_file)

ts.addDefaultSSLFiles()

ts.Disk.records_config.update(
    {
        'proxy.config.http.server_ports': '{0}:ssl:allow-plain'.format(ts.Variables.ssl_port),
        'proxy.config.ssl.server.cert.path': '{0}'.format(ts.Variables.SSLDir),
        'proxy.config.ssl.server.private_key.path': '{0}'.format(ts.Variables.SSLDir),
        'proxy.config.diags.debug.enabled': 0,
        'proxy.config.diags.debug.tags': 'ssl|http',
    })

ts.Disk.remap_config.AddLines(
    [
        'map / http://127.0.0.1:{0}'.format(server.Variables.http_port),
        'map /post http://127.0.0.1:{0}/post'.format(server.Variables.http_port),
    ])

ts.Disk.ssl_multicert_config.AddLine('dest_ip=* ssl_cert_name=server.pem ssl_key_name=server.key')

big_post_body = "0123456789" * 50000
big_post_body_file = open(os.path.join(Test.RunDirectory, "big_post_body"), "w")
big_post_body_file.write(big_post_body)
big_post_body_file.close()

# TLS curl should work of course
tr = Test.AddTestRun()
# Wait for the micro server
tr.Processes.Default.StartBefore(server, ready=When.PortOpen(server.Variables.http_port))
# Delay on readiness of our ssl ports
tr.Processes.Default.StartBefore(Test.Processes.ts)

tr.Processes.Default.Command = 'curl -o /dev/null -k --verbose -H "uuid: get" --ipv4 --http1.1 --resolve www.example.com:{}:127.0.0.1 https://www.example.com:{}/'.format(
    ts.Variables.ssl_port, ts.Variables.ssl_port)
tr.Processes.Default.ReturnCode = 0
tr.StillRunningAfter = server
tr.StillRunningAfter = ts
tr.Processes.Default.Streams.all = Testers.ContainsExpression("TLS", "Should negiotiate TLS")

# non-TLS curl should also work to the same port
tr2 = Test.AddTestRun()
tr2.Processes.Default.Command = 'curl --verbose --ipv4 --http1.1 -H "uuid: get" --resolve www.example.com:{}:127.0.0.1 http://www.example.com:{}'.format(
    ts.Variables.ssl_port, ts.Variables.ssl_port)
tr2.Processes.Default.ReturnCode = 0
tr2.StillRunningAfter = server
tr2.StillRunningAfter = ts
tr2.Processes.Default.Streams.all = Testers.ExcludesExpression("TLS", "Should not negiotiate TLS")

# Make sure a post > 32K works.  Early version forgot to free a reader which caused a stall once the initial buffer filled
# Seems like we needed to make a second resquest to trigger the issue
tr3 = Test.AddTestRun()
tr3.Processes.Default.Command = 'curl --verbose -d @big_post_body -H "uuid: post" --ipv4 --http1.1 --resolve www.example.com:{}:127.0.0.1 http://www.example.com:{}/post http://www.example.com:{}/post'.format(
    ts.Variables.ssl_port, ts.Variables.ssl_port, ts.Variables.ssl_port)
tr3.Processes.Default.ReturnCode = 0
tr3.StillRunningAfter = server
tr3.StillRunningAfter = ts
tr3.Processes.Default.Streams.all = Testers.ExcludesExpression("TLS", "Should not negiotiate TLS")



'''
'''
#  Licensed to the Apache Software Foundation (ASF) under one
#  or more contributor license agreements.  See the NOTICE file
#  distributed with this work for additional information
#  regarding copyright ownership.  The ASF licenses this file
#  to you under the Apache License, Version 2.0 (the
#  "License"); you may not use this file except in compliance
#  with the License.  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

import os
import sys

Test.Summary = '''
Test new log fields
'''

Test.SkipUnless(Condition.HasCurlFeature('http2'))

# ----
# Setup httpbin Origin Server
# ----
httpbin = Test.MakeHttpBinServer("httpbin")

# ----
# Setup ATS
# ----
ts = Test.MakeATSProcess("ts", enable_tls=True)

ts.addDefaultSSLFiles()

ts.Disk.records_config.update(
    {
        # 'proxy.config.diags.debug.enabled': 1,
        'proxy.config.ssl.server.cert.path': '{0}'.format(ts.Variables.SSLDir),
        'proxy.config.ssl.server.private_key.path': '{0}'.format(ts.Variables.SSLDir),
    })

ts.Disk.remap_config.AddLine('map http://127.0.0.1:{0} http://127.0.0.1:{1}/ip'.format(ts.Variables.port, httpbin.Variables.Port))

ts.Disk.remap_config.AddLine(
    'map https://127.0.0.1:{0} http://127.0.0.1:{1}/ip'.format(ts.Variables.ssl_port, httpbin.Variables.Port))

ts.Disk.remap_config.AddLine(
    'map https://reallyreallyreallyreallylong.com http://127.0.0.1:{1}/ip'.format(ts.Variables.ssl_port, httpbin.Variables.Port))

ts.Disk.ssl_multicert_config.AddLine('dest_ip=* ssl_cert_name=server.pem ssl_key_name=server.key')

ts.Disk.logging_yaml.AddLines(
    '''
logging:
  formats:
    - name: custom
      format: "%<ccid> %<ctid> %<cssn>"
  logs:
    - filename: test_new_log_flds
      format: custom
'''.split("\n"))

tr = Test.AddTestRun()
# Delay on readiness of ssl port
tr.Processes.Default.StartBefore(Test.Processes.ts)
tr.Processes.Default.StartBefore(httpbin, ready=When.PortOpen(httpbin.Variables.Port))
#
tr.Processes.Default.Command = 'curl "http://127.0.0.1:{0}" --verbose'.format(ts.Variables.port)
tr.Processes.Default.ReturnCode = 0

tr = Test.AddTestRun()
tr.Processes.Default.Command = 'curl "http://127.0.0.1:{0}" --verbose'.format(ts.Variables.port)
tr.Processes.Default.ReturnCode = 0

tr = Test.AddTestRun()
tr.Processes.Default.Command = 'curl "http://127.0.0.1:{0}" "http://127.0.0.1:{0}" --http1.1 --verbose'.format(ts.Variables.port)
tr.Processes.Default.ReturnCode = 0

tr = Test.AddTestRun()
tr.Processes.Default.Command = (
    'curl "https://127.0.0.1:{0}" "https://127.0.0.1:{0}" --http2 --insecure --verbose'.format(ts.Variables.ssl_port))
tr.Processes.Default.ReturnCode = 0

tr = Test.AddTestRun()
tr.Processes.Default.Command = (
    'curl "https://reallyreallyreallyreallylong.com:{0}" --http2 --insecure --verbose' +
    ' --resolve reallyreallyreallyreallylong.com:{0}:127.0.0.1').format(ts.Variables.ssl_port)
tr.Processes.Default.ReturnCode = 0

# Wait for log file to appear, then wait one extra second to make sure TS is done writing it.
#
test_run = Test.AddTestRun()
test_run.Processes.Default.Command = (
    os.path.join(Test.Variables.AtsTestToolsDir, 'condwait') + ' 60 1 -f ' +
    os.path.join(ts.Variables.LOGDIR, 'test_new_log_flds.log'))
test_run.Processes.Default.ReturnCode = 0

# Validate generated log.
#
tr = Test.AddTestRun()
observer_script = os.path.join(Test.TestDirectory, 'new_log_flds_observer.py')
log_path = os.path.join(ts.Variables.LOGDIR, 'test_new_log_flds.log')
tr.Processes.Default.Command = f'{sys.executable} {observer_script} < {log_path}'
tr.Processes.Default.ReturnCode = 0



'''
Verify correct behavior of regex_map in remap.config.
'''
#  Licensed to the Apache Software Foundation (ASF) under one
#  or more contributor license agreements.  See the NOTICE file
#  distributed with this work for additional information
#  regarding copyright ownership.  The ASF licenses this file
#  to you under the Apache License, Version 2.0 (the
#  "License"); you may not use this file except in compliance
#  with the License.  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

Test.Summary = '''
Test minimum rules on load - fail on missing file.
'''
ts = Test.MakeATSProcess("ts")
ts.Disk.remap_config.AddLine(f"")  # empty file
ts.Disk.records_config.update({'proxy.config.url_remap.min_rules_required': 1})
ts.ReturnCode = 33  # expect to Emergency fail due to empty "remap.config".
ts.Ready = 0

tr = Test.AddTestRun("test")

# We have to wait upon TS to emit the expected log message, but it cannot be
# the ts Ready criteria because autest might detect the process going away
# before it detects the log message. So we add a separate process that waits
# upon the log message.
watcher = Test.Processes.Process("watcher")
watcher.Command = "sleep 10"
watcher.Ready = When.FileContains(ts.Disk.diags_log.Name, "remap.config failed to load")
watcher.StartBefore(ts)

tr.Processes.Default.Command = "echo howdy"
tr.TimeOut = 5
tr.Processes.Default.StartBefore(watcher)
